/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <ds/asio/resource.h>
#include <ds/conf.h>
#include <ds/json.h>
#include <variant>
#include "http_client.h"
#include "model/apartment.h"
#include "vdc.h"

namespace dss {

struct PutResource {
  ds::JsonFragment value;

  PutResource(const PutResource&) = delete;
  PutResource(PutResource&&) = default;
  PutResource(ds::JsonFragment&& value) : value(std::move(value)){};
};
struct DeleteResource {};
struct PatchResource {
  ds::JsonFragment diff;

  PatchResource(const PatchResource&) = delete;
  PatchResource(PatchResource&&) = default;
  PatchResource(ds::JsonFragment&& diff) : diff(std::move(diff)){};
};

typedef std::variant<PutResource, DeleteResource, PatchResource> ResourceNotification;

struct CollectionNotification {
  std::string id;
  ResourceNotification notification;
};

BOOST_STRONG_TYPEDEF(std::string, BackendVdcId);
BOOST_STRONG_TYPEDEF(std::string, BackendVdcDeviceId);

struct BackendVdcsReregisterHooks {
  std::function<void()> succeeded{};
  std::function<void()> attemptFailed{};
};

class BackendVdcs {
public:
  virtual ~BackendVdcs();

  virtual void notify(const std::string& path, ResourceNotification&&) = 0;

  virtual void reregister(BackendVdcsReregisterHooks&& hooks = {}) = 0;
  virtual void reregisterDevices(const dsuid_t& dsuid) = 0;
  virtual StructureQueryBusInterface& structureQueryBusInterface() = 0;
  virtual StructureModifyingBusInterface& structureModifyingBusInterface() = 0;

  virtual bool isBackendVdcDevice(const dsuid_t&) const = 0;
  virtual BackendVdcId vdcIdForDeviceDsuid(const dsuid_t&) const = 0;
  virtual BackendVdcId vdcIdForDsuid(const dsuid_t&) const = 0;
  virtual std::string vdcDsuidForId(const BackendVdcId& vdcId) const = 0;

  virtual bool hasBackend() const = 0;

  // Calls http request with added base url, authentication and language headers
  virtual ds::JsonFragment http(HttpRequest&& x, const std::optional<std::locale>& = {},
                                ds::LogSeverity logSeverity = ds::LogSeverity::NOTICE) const = 0;
};

std::unique_ptr<BackendVdcs> makeBackendVdcs(ds::asio::ResourceContext&, Apartment&, EventInterpreter&,
                                             SimpleHttpClient&);

} // namespace dss
