#pragma once
#include "businterface.h"
#include "ds485/dsbusinterface.h"
#include "messages/vdc-messages.pb.h"

namespace dss {

class DSS;
class ModelMaintenance;

class Bus : public BusInterface, public StructureQueryBusInterface, StructureModifyingBusInterface {
public:
  Bus(DSS& dss, ModelMaintenance& modelMaintenance);

  DSBusInterface& dsBus() { return m_ds485Bus; }
  StructureQueryBusInterface& dsmQueryBus(const dsuid_t& dsmId);
  StructureModifyingBusInterface& dsmModifyBus(const dsuid_t& dsmId);

  DeviceBusInterface* getDeviceBusInterface() override { return m_ds485Bus.getDeviceBusInterface(); }
  StructureQueryBusInterface* getStructureQueryBusInterface() override { return this; }
  MeteringBusInterface* getMeteringBusInterface() override { return m_ds485Bus.getMeteringBusInterface(); }
  StructureModifyingBusInterface* getStructureModifyingBusInterface() override { return this; }
  ActionRequestInterface* getActionRequestInterface() override { return m_ds485Bus.getActionRequestInterface(); }
  void setBusEventSink(BusEventSink* x) override { return m_ds485Bus.setBusEventSink(x); }
  const std::string getConnectionURI() override { return m_ds485Bus.getConnectionURI(); }
  void add(StructureModifyingBusInterface& x) override { m_modifies.push_back(std::ref(x)); }

  std::string name() const override;

  // StructureQueryBusInterface
  std::vector<DSMeterSpec_t> getBusMembers() override { return m_ds485Query.getBusMembers(); }
  DSMeterSpec_t getDSMeterSpec(const dsuid_t& dsm) override { return dsmQueryBus(dsm).getDSMeterSpec(dsm); }
  std::vector<int> getZones(const dsuid_t& dsm) override { return dsmQueryBus(dsm).getZones(dsm); }
  std::vector<DeviceSpec> getDevicesInZone(const dsuid_t& dsm, const int zone, bool complete = true) override {
    return dsmQueryBus(dsm).getDevicesInZone(dsm, zone, complete);
  }
  std::vector<DeviceSpec> getInactiveDevicesInZone(const dsuid_t& dsm, const int zone) override {
    return dsmQueryBus(dsm).getInactiveDevicesInZone(dsm, zone);
  }
  std::vector<GroupSpec_t> getGroups(const dsuid_t& dsm, const int zone) override {
    return dsmQueryBus(dsm).getGroups(dsm, zone);
  }
  std::vector<ClusterSpec_t> getClusters(const dsuid_t& dsm) override { return dsmQueryBus(dsm).getClusters(dsm); }
  std::vector<CircuitPowerStateSpec_t> getPowerStates(const dsuid_t& dsm) override {
    return dsmQueryBus(dsm).getPowerStates(dsm);
  }
  std::vector<std::pair<int, int>> getLastCalledScenes(const dsuid_t& dsm, const int zone) override {
    return dsmQueryBus(dsm).getLastCalledScenes(dsm, zone);
  }
  std::bitset<7> getZoneStates(const dsuid_t& dsm, const int zone) override {
    return dsmQueryBus(dsm).getZoneStates(dsm, zone);
  }
  bool getEnergyBorder(const dsuid_t& dsm, int& lower, int& upper) override {
    return dsmQueryBus(dsm).getEnergyBorder(dsm, lower, upper);
  }
  DeviceSpec deviceGetSpec(devid_t id, dsuid_t dsm) override { return dsmQueryBus(dsm).deviceGetSpec(id, dsm); }
  std::string getSceneName(dsuid_t dsm, boost::shared_ptr<Group> group, const uint8_t scene) override {
    return dsmQueryBus(dsm).getSceneName(dsm, group, scene);
  }
  DSMeterHash_t getDSMeterHash(const dsuid_t& dsm) override { return dsmQueryBus(dsm).getDSMeterHash(dsm); }
  void getDSMeterState(const dsuid_t& dsm, uint8_t* state) override {
    return dsmQueryBus(dsm).getDSMeterState(dsm, state);
  }
  ZoneHeatingConfigSpec_t getZoneHeatingConfig(const dsuid_t& dsm, const uint16_t zone) override {
    return dsmQueryBus(dsm).getZoneHeatingConfig(dsm, zone);
  }
  ZoneHeatingInternalsSpec_t getZoneHeatingInternals(const dsuid_t& dsm, const uint16_t zone) override {
    return dsmQueryBus(dsm).getZoneHeatingInternals(dsm, zone);
  }
  ZoneHeatingStateSpec_t getZoneHeatingState(const dsuid_t& dsm, const uint16_t zone) override {
    return dsmQueryBus(dsm).getZoneHeatingState(dsm, zone);
  }
  ZoneHeatingOperationModeSpec_t getZoneHeatingOperationModes(const dsuid_t& dsm, const uint16_t zone) override {
    return dsmQueryBus(dsm).getZoneHeatingOperationModes(dsm, zone);
  }
  dsuid_t getZoneSensor(const dsuid_t& dsm, const uint16_t zone, SensorType sensor) override {
    return dsmQueryBus(dsm).getZoneSensor(dsm, zone, sensor);
  }
  void getZoneSensorValue(const dsuid_t& dsm, const uint16_t zone, SensorType sensor, uint16_t* value,
                          uint32_t* age) override {
    return dsmQueryBus(dsm).getZoneSensorValue(dsm, zone, sensor, value, age);
  }
  int getDevicesCountInZone(const dsuid_t& dsm, const int zone, bool onlyActive) override {
    return dsmQueryBus(dsm).getDevicesCountInZone(dsm, zone, onlyActive);
  }
  void protobufMessageRequest(const dsuid_t dsm, const uint16_t requestSize, const uint8_t* request,
                              uint16_t* responseSize, uint8_t* response) override {
    return dsmQueryBus(dsm).protobufMessageRequest(dsm, requestSize, request, responseSize, response);
  }

  // StructureModifyingBusInterface
  void setZoneID(const dsuid_t& dsm, const devid_t device, const int zone) override {
    dsmModifyBus(dsm).setZoneID(dsm, device, zone);
  }
  void createZone(const dsuid_t& dsm, const int zone) override { dsmModifyBus(dsm).createZone(dsm, zone); }
  void removeZone(const dsuid_t& dsm, const int zone) override { dsmModifyBus(dsm).removeZone(dsm, zone); }
  void addToGroup(const dsuid_t& dsm, const int group, const int device) override {
    dsmModifyBus(dsm).addToGroup(dsm, group, device);
  }
  void removeFromGroup(const dsuid_t& dsm, const int group, const int device) override {
    dsmModifyBus(dsm).removeFromGroup(dsm, group, device);
  }
  void removeDeviceFromDSMeter(const dsuid_t& dsm, const int device) override {
    dsmModifyBus(dsm).removeDeviceFromDSMeter(dsm, device);
  }
  void removeDeviceFromDSMeters(const dsuid_t& device) override;
  void sceneSetName(uint16_t zone, uint8_t group, uint8_t scene, const std::string& name) override;
  void deviceSetName(dsuid_t dsm, devid_t device, const std::string& name) override {
    dsmModifyBus(dsm).deviceSetName(dsm, device, name);
  }
  void meterSetName(dsuid_t dsm, const std::string& name) override { dsmModifyBus(dsm).meterSetName(dsm, name); }
  void createGroup(uint16_t zone, uint8_t group, ApplicationType application, uint32_t applicationConfig,
                   const std::string& name) override {
    for (const auto& modify : m_modifies) {
      modify.get().createGroup(zone, group, application, applicationConfig, name);
    }
  }
  void removeGroup(uint16_t zone, uint8_t group) override {
    for (const auto& modify : m_modifies) {
      modify.get().removeGroup(zone, group);
    }
  }
  void groupSetApplication(uint16_t zone, uint8_t group, ApplicationType application,
                           uint32_t applicationConfig) override {
    for (const auto& modify : m_modifies) {
      modify.get().groupSetApplication(zone, group, application, applicationConfig);
    }
  }
  void groupSetName(uint16_t zone, uint8_t group, const std::string& name) override {
    for (const auto& modify : m_modifies) {
      modify.get().groupSetName(zone, group, name);
    }
  }
  void createCluster(uint8_t group, ApplicationType application, uint32_t applicationConfig,
                     const std::string& name) override {
    for (const auto& modify : m_modifies) {
      modify.get().createCluster(group, application, applicationConfig, name);
    }
  }
  void removeCluster(uint8_t cluster) override {
    for (const auto& modify : m_modifies) {
      modify.get().removeCluster(cluster);
    }
  }
  void clusterSetName(uint8_t cluster, const std::string& name) override {
    for (const auto& modify : m_modifies) {
      modify.get().clusterSetName(cluster, name);
    }
  }
  void clusterSetApplication(uint8_t cluster, ApplicationType application, uint32_t applicationConfig) override {
    for (const auto& modify : m_modifies) {
      modify.get().clusterSetApplication(cluster, application, applicationConfig);
    }
  }
  void clusterSetProperties(uint8_t cluster, uint16_t location, uint16_t floor, uint16_t protectionClass) override {
    for (const auto& modify : m_modifies) {
      modify.get().clusterSetProperties(cluster, location, floor, protectionClass);
    }
  }
  void clusterSetLockedScenes(uint8_t cluster, const std::vector<int>& lockedScenes) override {
    for (const auto& modify : m_modifies) {
      modify.get().clusterSetLockedScenes(cluster, lockedScenes);
    }
  }
  void clusterSetConfigurationLock(uint8_t cluster, bool lock) override {
    for (const auto& modify : m_modifies) {
      modify.get().clusterSetConfigurationLock(cluster, lock);
    }
  }
  void clusterSetOperationLock(uint8_t cluster, bool lock) override {
    for (const auto& modify : m_modifies) {
      modify.get().clusterSetOperationLock(cluster, lock);
    }
  }
  void setButtonSetsLocalPriority(const dsuid_t& dsm, const devid_t device, bool setsPriority) override {
    dsmModifyBus(dsm).setButtonSetsLocalPriority(dsm, device, setsPriority);
  }
  void setButtonCallsPresent(const dsuid_t& dsm, const devid_t device, bool callsPresent) override {
    dsmModifyBus(dsm).setButtonCallsPresent(dsm, device, callsPresent);
  }
  void setZoneHeatingConfig(const dsuid_t& dsm, const uint16_t zone, const ZoneHeatingConfigSpec_t spec) override {
    dsmModifyBus(dsm).setZoneHeatingConfig(dsm, zone, spec);
  }
  void setZoneHeatingOperationModes(const dsuid_t& dsm, const uint16_t zone,
                                    const ZoneHeatingOperationModeSpec_t spec) override {
    dsmModifyBus(dsm).setZoneHeatingOperationModes(dsm, zone, spec);
  }
  void setZoneSensor(const uint16_t zone, SensorType sensorType, const dsuid_t& sensorDSUID) override {
    for (const auto& modify : m_modifies) {
      modify.get().setZoneSensor(zone, sensorType, sensorDSUID);
    }
  }
  void resetZoneSensor(const uint16_t zone, SensorType sensorType) override {
    for (const auto& modify : m_modifies) {
      modify.get().resetZoneSensor(zone, sensorType);
    }
  }
  void setCircuitPowerStateConfig(const dsuid_t& dsm, const int index, const int setThreshold,
                                  const int resetThreshold) override {
    dsmModifyBus(dsm).setCircuitPowerStateConfig(dsm, index, setThreshold, resetThreshold);
  }
  void setProperty(const dsuid_t& dsm,
                   const ::google::protobuf::RepeatedPtrField<::vdcapi::PropertyElement>& properties) override {
    dsmModifyBus(dsm).setProperty(dsm, properties);
  }
  vdcapi::Message getProperty(const dsuid_t& dsm,
                              const ::google::protobuf::RepeatedPtrField<::vdcapi::PropertyElement>& query) override {
    return dsmModifyBus(dsm).getProperty(dsm, query);
  }
  void setFireMuteEnabled(dsuid_t dsm, bool x) override { return dsmModifyBus(dsm).setFireMuteEnabled(dsm, x); }
  void removeVdc(const dsuid_t& vdsmID, const dsuid_t& vdcID) override {
    return dsmModifyBus(vdsmID).removeVdc(vdsmID, vdcID);
  }

  void clusterSetStopTimeout(uint8_t cluster, uint8_t timeoutSeconds) override {
    for (const auto& modify : m_modifies) {
      modify.get().clusterSetStopTimeout(cluster, timeoutSeconds);
    }
  };

  void zoneSetStopTimeout(uint16_t zone, uint8_t group, uint8_t timeoutSeconds) override {
    for (const auto& modify : m_modifies) {
      modify.get().zoneSetStopTimeout(zone, group, timeoutSeconds);
    }
  }

private:
  DSS& m_dss;
  DSBusInterface m_ds485Bus;
  StructureQueryBusInterface& m_ds485Query;
  StructureModifyingBusInterface& m_ds485Modify;
  std::vector<std::reference_wrapper<StructureQueryBusInterface>> m_queries;
  std::vector<std::reference_wrapper<StructureModifyingBusInterface>> m_modifies;
};

} // namespace dss
