/*
    Copyright (c) 2018 digitalSTROM AG, Zurich, Switzerland

    This file is part of dSS.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dSS. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <boost/make_shared.hpp>
#include <chrono>
#include <functional>
#include "cache-db.h"
#include "device-cache.h"
#include "event.h"

namespace dss {

class Apartment;
class Device;
class HardwareAbstraction;

class CacheUpdatePlugin : public EventInterpreterPlugin {
public:
  static constexpr const char* CacheUpdateEvent = "device_scene_cache_update";
  static constexpr const char* CacheUpdateEventFast = "device_scene_cache_update_fast";
  static constexpr const char* CacheUpdateEventResume = "device_scene_cache_update_resume";
  static constexpr const int CacheUpdateDefaultBackoffInSec = 2;

  CacheUpdatePlugin(const Apartment* apt, HardwareAbstraction* hal, EventInterpreter& interpreter,
                    boost::shared_ptr<PropertySystem> propSystem, boost::shared_ptr<EventQueue> eventQueue,
                    boost::shared_ptr<EventRunner> eventRunner);
  virtual ~CacheUpdatePlugin();
  virtual void subscribe();
  virtual void handleEvent(Event& _event, const EventSubscription& _subscription);

  bool isUpdateRunning() { return m_updateInProgress; }

private:
  void scanCache();
  void reschedule();

  bool updateIfNotCached(const Dsuid& dsuid, DeviceCache& cache, std::string path);
  bool updateIfNotCached(const Dsuid& dsuid, SceneFlags& flags, std::string path);
  bool updateIfNotCached(const Dsuid& dsuid, SceneEntry& entry, std::string path);
  bool updateIfNotCached(const Dsuid& dsuid, DeviceSceneCache& sceneCache, std::string path);

  const Apartment* m_apartment;
  HardwareAbstraction* m_hal;
  PropertyNodePtr m_scanActive;
  boost::shared_ptr<PropertySystem> m_pPropSystem;
  boost::shared_ptr<EventQueue> m_pEventQueue;
  boost::shared_ptr<EventRunner> m_pEventRunner;

  bool m_updateInProgress;

  std::optional<Dsuid> m_currentDev;
  std::string m_currentPath;
  DeviceCache m_currentEntry;

  int m_hitBackoffInSec;
};

} // namespace dss
