/*
 *  Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <model/modelconst.h>
#include <map>
#include <optional>
#include "ds/date-time.h"
#include "ds/enum.h"
#include "ds/json.h"

DS_REFLECTED_ENUM(
    (dss), DeviceProperty,
    (outputMode)(switchedThreshold)(switchedThresholdTwoStep0)(switchedThresholdTwoStep1)(switchedThreshold3Step0)(switchedThreshold3Step1)(switchedThreshold3Step2)(dimTime0Up)(dimTime0Down)(dimTime1Up)(dimTime1Down)(dimTime2Up)(dimTime2Down)(
        dimStepUp)(dimStepDown)(dimStepNum)(dimStepLen)(dimStepPause)(posTimeUp)(posTimeDown)(minDim));
DS_REFLECTED_ENUM((dss), DimTime, (noDimTime)(dimTime0)(dimTime1)(dimTime2)(notSupported));
DS_REFLECTED_ENUM(
    (dss), LedConfig, (defaultConfig)(ledCon0)(ledCon1)(ledCon2)(ledCon3)(ledCon4)(ledCon5)(ledCon6));

namespace dss {

struct SceneFlags {
  std::optional<bool> dontCare;
  std::optional<bool> ignoreLocalPrio;
  std::optional<bool> specialHandling;
  std::optional<bool> blink;
  std::optional<LedConfig> ledConfig;
  std::optional<DimTime> dimTime;
};

struct SceneChannel {
  std::optional<double> value;
  std::optional<bool> dontCare;
  std::vector<std::optional<int>> registerValues;
};

struct SceneEntry {
  std::optional<ds::DateTime> lastChanged;
  SceneFlags flags;
  std::optional<std::string> action;
  std::map<ChannelType, SceneChannel> channels;
};

struct DeviceSceneCache {
  std::optional<int> changeCounter;
  std::map<int, SceneEntry> sceneTable; //<sceneId, SceneEntry>
};

struct DeviceCache {
  std::optional<ds::DateTime> lastChanged;
  struct DeviceSceneCache scene;
  std::map<DeviceProperty, std::optional<int>> property;
};

} // namespace dss

DS_JSON_DECLARE_STRUCT((dss), SceneFlags, (dontCare)(ignoreLocalPrio)(specialHandling)(blink)(ledConfig)(dimTime));
DS_JSON_DECLARE_STRUCT((dss), SceneChannel, (value)(registerValues)(dontCare));
DS_JSON_DECLARE_STRUCT((dss), SceneEntry, (lastChanged)(flags)(action)(channels));
DS_JSON_DECLARE_STRUCT((dss), DeviceSceneCache, (changeCounter)(sceneTable));
DS_JSON_DECLARE_STRUCT((dss), DeviceCache, (lastChanged)(scene)(property));
