/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once
#include <ds/reflect.h>
#include <gsl.h>
#include <boost/serialization/strong_typedef.hpp>
#include <map>
#include <optional>
#include <variant>

namespace dsApi {

BOOST_STRONG_TYPEDEF(gsl::cstring_span, LiteralRouteSegment);
BOOST_STRONG_TYPEDEF(gsl::cstring_span, ArgRouteSegment);
BOOST_STRONG_TYPEDEF(gsl::cstring_span, WildcardArgRouteSegment);
using RouteSegment = std::variant<LiteralRouteSegment, ArgRouteSegment, WildcardArgRouteSegment>;
std::ostream& operator<<(std::ostream&, const RouteSegment&);

struct MatchRouteResult;

struct RouteSegmentsView {
  std::vector<RouteSegment> segments;
  RouteSegmentsView() = default;
  RouteSegmentsView(const gsl::cstring_span path);

  MatchRouteResult match(const gsl::cstring_span path) const;
};

struct MatchRouteResult {
  bool matched{false};
  std::map<std::string, std::string> args;

  explicit operator bool() const { return matched; }
};

/// Matches route pattern against path and parses arguments.
///
/// Inspired by System.Web.Routing.Route in dotnet.
///
/// Each route pattern segment is delimited by '/' character.
/// When a segment is enclosed in braces ('{' and '}'), the segment is an argument.
/// Routing retrieves values from request and assigns it to args by argument name.
///
/// Example route patterns:
/// - "/apartments/{apartmentId}
/// - "/apartments/{apartmentId}/status
/// - "/apartments/{apartmentId}/zones
/// - "/apartments/{apartmentId}/zones/{zoneId}
/// - "/apartments/{apartmentId}/zones/{zoneId}/status
/// - "/apartments/{apartmentId}/{*path}
MatchRouteResult matchRoute(const gsl::cstring_span routePattern, const gsl::cstring_span path);

} // namespace dsApi

DS_REFLECT_DECLARE_STRUCT((dsApi), RouteSegmentsView, (segments));
DS_REFLECT_DECLARE_STRUCT((dsApi), MatchRouteResult, (matched)(args));
