#pragma once

#include "text-fwd.h"

#include <ds/json.h>
#include <variant>

namespace dsApi {

/// User visible translatable text.
struct Text {
  /// Translation domain identifying catalog with translations.
  /// The name of .po / .json file with translations.
  ///
  ///
  /// Empty -> text message is not translatable (e.g. user entered text).
  std::string domain;

  /// Key for message lookup in domain catalog.
  ///
  /// The lookup result is the string presented to user.
  std::string message;

  /// Optional disambiguation context for lookup.
  std::string context;
};

} // namespace dsApi

DS_JSON_DECLARE_STRUCT((dsApi), Text, (domain)(message)(context));

// `dsTranslation` is used to harvest translations from source code.
inline dsApi::Text dsApiTranslation(std::string &&domain, std::string &&context, std::string &&message) {
  return {.domain = std::move(domain), .message = std::move(message), .context = std::move(context)};
}
