#include "backend-stream-client.h"

#include <ds/conf.h>
#include <ds/system.h>
#include "wss-client.h"

namespace dsConnector {

std::unique_ptr<dsConnector::StreamClient> makeBackendStreamClient(boost::asio::io_service &io, ds::System &system,
                                                                   boost::asio::ssl::context &sslContext) {
  // We could eliminate sslContext from argument.
  // But we would need an option to attach it to returned std::unique_ptr<dsConnector::StreamClient>.
  return dsConnector::makeWSSClient(io, sslContext, system.conf().readOr<std::string>("backendConnectorUrl", [&] {
    const auto release = system.release();
    switch (release) {
      case ds::Release::devel: return "wss://dsbcdev.devapi.digitalstrom.com:8256/dss";
      case ds::Release::testing: return "wss://dsbcalpha.devapi.digitalstrom.com:8256/dss";
      case ds::Release::fieldtest: return "wss://dsbcbeta.devapi.digitalstrom.com:8256/dss";
      case ds::Release::production: return "wss://dsbc.devapi.digitalstrom.com:8256/dss";
    }
    DS_ASSERT("Unknown release", release);
    return ""; // To work around -Werror=return-type in gcc 7.3
  }()));
}

} // namespace dsConnector
