/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include "client-bus-fwd.h"

#include <ds-api/event-fwd.h>
#include <ds-api/rest.h>
#include <ds/conf.h>
#include <ds/ds-signal.h>
#include <boost/noncopyable.hpp>
#include <string>
#include "protocol-fwd.h"
#include "stream.h"
#include "subscriptions.h"

namespace dsConnector {

class ClientBusHook : public dsApi::AsyncCallee {
public:
  using Callback = std::function<void(std::variant<ds::JsonFragment, std::exception> &&)>;
  virtual ~ClientBusHook() = default;

  virtual Authenticate authenticate() = 0;

  // from ds::AsyncCallee
  void asyncInvoke(dsApi::HttpRequest &&, std::unique_ptr<ds::Promise<dsApi::HttpResult>> &&) override = 0;
};

class ClientBus : public Subscriber, public dsApi::AsyncCallee {
public:
  virtual ~ClientBus() = default;

  // thread safe
  virtual bool isConnected() const = 0;

  // thread safe
  virtual void setActive(bool) = 0;

  ds::Slots<bool> &isConnectedChanged() { return m_isConnectedChanged; }

  // from Subscriber
  void event(dsApi::Event &&) override = 0;

  // from ds::AsyncCallee
  void asyncInvoke(dsApi::HttpRequest &&, std::unique_ptr<ds::Promise<dsApi::HttpResult>> &&) override = 0;

protected:
  ds::Signal<bool> m_isConnectedChanged;
};

///
/// Maintains stream client connection and provides bus interface for it.
///
std::unique_ptr<ClientBus> makeClientBus(boost::asio::io_service &, ds::Conf &dsConf, StreamClient &, ClientBusHook &,
                                         Subscriptions &);

} // namespace dsConnector
