/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include "protocol-fwd.h"

#include <ds-api/event.h>
#include <ds-api/rest.h>
#include <ds/clock.h>
#include <ds/enum.h>
#include <ds/json.h>
#include <ds/reflect.h>
#include <optional>
#include <string>

namespace dsConnector {

BOOST_STRONG_TYPEDEF(std::string, RequestId);

struct Authenticate {
  std::string client;
  std::string secret;
  std::string apartmentId;
};

struct Authenticated {
  std::string _dummy; // reflection framework requires at least one field
};

struct Ping {
  std::string id;
};

struct Pong {
  std::string id;
};

struct Request {
  RequestId id;
  std::string path;
  dsApi::HttpArgs args;
};

struct Response {
  RequestId id;
  dsApi::HttpResult result;
};

struct Event {
  dsApi::EventType type;
  std::string path;
  ds::JsonFragment data;
};

struct Subscribe {
  dsApi::EventType type;
  ds::Clock::duration timeout;
};

struct Message {
  std::optional<Authenticate> authenticate;
  std::optional<Authenticated> authenticated;
  std::optional<Ping> ping;
  std::optional<Pong> pong;
  std::optional<Request> request;
  std::optional<Response> response;
  std::optional<Event> event;
  std::optional<Subscribe> subscribe;
};

} // namespace dsConnector

DS_JSON_DECLARE_STRUCT((dsConnector), Authenticate, (client)(secret)(apartmentId));
DS_JSON_DECLARE_STRUCT((dsConnector), Authenticated, (_dummy));
DS_JSON_DECLARE_STRUCT((dsConnector), Ping, (id));
DS_JSON_DECLARE_STRUCT((dsConnector), Pong, (id));
DS_JSON_DECLARE_STRUCT((dsConnector), Request, (id)(path)(args));
DS_JSON_DECLARE_STRUCT((dsConnector), Response, (id)(result));
DS_JSON_DECLARE_STRUCT((dsConnector), Event, (type)(path)(data));
DS_JSON_DECLARE_STRUCT((dsConnector), Subscribe, (type)(timeout));
DS_JSON_DECLARE_STRUCT((dsConnector), Message,
                       (authenticate)(authenticated)(ping)(pong)(request)(response)(event)(subscribe));
