/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/reflect.h>
#include <gsl.h>
#include <boost/asio/io_service.hpp>
#include <boost/system/error_code.hpp>
#include <functional>
#include <memory>
#include <string>

namespace dsConnector {

/// Bidirection stream of framed messages.
/// New stream is created for every new connection.
class Stream {
public:
  /// Destroyed stream must not call any further callbacks
  virtual ~Stream() = default;
  using WriteHandler = std::function<void(boost::system::error_code)>;
  using ReadHandler = std::function<void(boost::system::error_code, std::string &&)>;
  virtual void asyncWrite(std::string_view buffer, WriteHandler &&) = 0;
  virtual void asyncRead(ReadHandler &&) = 0;
  virtual std::string str() const = 0;
};
std::ostream &operator<<(std::ostream &, const Stream &);

/// Stream client connecting streams to server.
class StreamClient {
public:
  virtual ~StreamClient() = default;
  using ConnectHandler = std::function<void(boost::system::error_code)>;
  virtual std::unique_ptr<Stream> asyncConnect(ConnectHandler &&) = 0;
  virtual std::string str() const = 0;
};
std::ostream &operator<<(std::ostream &, const StreamClient &);

} // namespace dsConnector
