/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include "subscriptions-fwd.h"

#include <ds-api/event.h>
#include <ds/clock.h>
#include <boost/asio/io_service.hpp>

namespace dsConnector {

/// Subscriber consumes subscribed events
struct Subscriber {
  virtual ~Subscriber() {}
  virtual void event(dsApi::Event&&) = 0;
};

struct Subscription {
  dsApi::EventType type;
  ds::Clock::time_point expiresAt;
  Subscriber* sink;
};

struct SubscriptionsHooks {
  // primarily to support unit tests
  std::function<void(dsApi::Event&&)> discardedEvent{[](auto&&) {}};
};

struct Subscriptions : public dsApi::EventsSink {
  virtual ~Subscriptions() {}
  virtual void subscribe(Subscription&&) = 0;

  // thread safe
  void event(dsApi::Event&&) override = 0;
};

std::unique_ptr<Subscriptions> makeSubscriptions(boost::asio::io_service& io, SubscriptionsHooks&& = {});

} // namespace dsConnector

DS_REFLECT_DECLARE_STRUCT((dsConnector), Subscription, (type)(expiresAt)(sink));
