/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/reflect.h>
#include <ds/str.h>
#include <boost/asio/ssl/context.hpp>
#include "stream.h"

namespace dsConnector {

/// Parsed websocket url (ssl only for now).
struct WSSUrl {
  std::string host;
  std::string port{"443"};
  std::string target;
};

/// Parses string like wws://host:port/target into WWSUrl
WSSUrl parseWSSUrl(gsl::cstring_span);

std::unique_ptr<StreamClient> makeWSSClient(boost::asio::io_service &, boost::asio::ssl::context &, WSSUrl &&);
std::unique_ptr<StreamClient> makeWSSClient(boost::asio::io_service &, boost::asio::ssl::context &,
                                            const gsl::cstring_span url);

} // namespace dsConnector

DS_REFLECT_STRUCT((dsConnector), WSSUrl, (host)(port)(target));
