#pragma once

#include <ds-api/service-fwd.h>
#include <ds-connector/client-bus-fwd.h>
#include <ds-connector/subscriptions-fwd.h>
#include <ds-model/apartment-api.h>
#include <ds-model/model.h>
#include <ds/asio/resource.h>
#include <ds/conf.h>
#include <boost/noncopyable.hpp>
#include "handler/system_states.h"
#include "model/apartment.h"

///
/// Glue between dss model and ds model
///

template <>
struct DSToEnum<dsModel::ZoneApplicationType, dss::ApplicationType, void> {
  static std::optional<dsModel::ZoneApplicationType> tryTo(dss::ApplicationType x);
};

template <>
struct DSToEnum<dsModel::StateType, dss::BinaryInputType, void> {
  static std::optional<dsModel::StateType> tryTo(dss::BinaryInputType x);
};

template <>
struct DSToEnum<dsModel::OutputType, dss::ChannelType, void> {
  static std::optional<dsModel::OutputType> tryTo(dss::ChannelType x);
};

template <>
struct DSToEnum<dsModel::MeasurementUsage, int, void> {
  static std::optional<dsModel::MeasurementUsage> tryTo(int x);
};

namespace dss {
struct DecodedMeteringId {
  dsModel::Origin origin{};
  std::string meteringType{};
};

struct DecodedApplicationId {
  int zoneId{};
};

DecodedMeteringId decodeId(const dsModel::MeteringId&);
DecodedApplicationId decodeId(const dsModel::ApplicationId&);

std::vector<dsModel::ClusterId> resourceIds(const std::vector<dsModel::Cluster>&);

dsModel::Sensor makeDsModelSensor();
dsModel::Button makeDsModelButton();
dsModel::State makeDsModelState();
dsModel::Output makeDsModelOutput();

dsModel::ZoneApplicationType zoneApplication(const int& application);
dsModel::OutputMode outputMode(const int& mode);
std::optional<int> outputLevels(const int& mode);
dsModel::ButtonMode buttonMode(const ButtonInputMode& mode);

dsModel::MeasurementType measurementType(const SensorType& type);
dsModel::ButtonType buttonType(const int& type, const int& activeGroup);
dsModel::StateType stateType(const BinaryInputType& type);
dsModel::OutputType outputType(ChannelType type);

dsModel::ShadeAttributes shadeAttributes(const Device& device);
dsModel::ShadeDeviceType shadeDeviceType(const Device& device);

std::vector<dsModel::Sensor> getSensors(const Device& device);
std::vector<dsModel::Button> getButtons(const Device& device);
std::vector<dsModel::State> getStates(const Device& device);
std::vector<dsModel::Output> getOutputs(const Device& device, std::optional<std::vector<double>> levelThresholds);

class DsModel : public dsModel::ApartmentApi {
public:
  virtual void handleEvent(const Event& event) = 0; // for mocking purposes
};

struct DsModelHooks {
  // to support unit tests
  std::function<void()> apartmentStructurePolled{[] {}};
  std::function<void()> apartmentStatusPolled{[] {}};
};

std::unique_ptr<DsModel> makeDsModel(boost::asio::io_service&, ds::Conf&, dsConnector::ClientBus&, Apartment&,
                                     EventInterpreter&, dsConnector::Subscriptions&, ds::asio::ResourceContext&,
                                     DsModelHooks&& = {});

} // namespace dss

DS_REFLECT_STRUCT((dss), DecodedMeteringId, (origin)(meteringType));
