#pragma once

#include "model-fwd.h"
#include "model-ids.h"
#include "scenarios-ids.h"

#include <ds-api/event.h>
#include <ds-api/rest.h>
#include <ds/clock.h>
#include <ds/enum.h>
#include <ds/reflect.h>
#include <ds/str.h>
#include <boost/serialization/strong_typedef.hpp>
#include <memory>
#include <string>
#include <vector>
#include "scenarios.h"

///
/// Ds-model.
///
/// Reference documentation:
/// https://git.digitalstrom.org/digitalstrom-documentation/digitalstrom-documentation/blob/master/datamodel/
///

DS_REFLECTED_ENUM(
    (dsModel), MeasurementType,
    (none)(activePower)(outputCurrent)(energyMeter)(temperatureIndoors)(temperatureOutdoors)(brightnessIndoors)(
        brightnessOutdoors)(humidityIndoors)(humidityOutdoors)(airPressure)(windGustSpeed)(windGustDirection)(
        windAverageSpeed)(windDirection)(precipitation)(carbonDioxide)(carbonMonoxide)(soundPressureLevel)(
        tControlSetPoint)(tControlVariable)(status)(reserved1)(reserved2)(outputCurrent16A)(apparentPower)(temperature)(
        brightness)(humidity)(generatedActivePower)(generatedEnergyMeter)(waterQuantity)(waterFlowRate)(length)(mass)(
        time)(notUsed)(unknownType));

DS_REFLECTED_ENUM((dsModel), MeasurementUsage, (none)(zone)(outdoor)(settings)(device)(deviceLastRun)(deviceAverage));

DS_REFLECTED_ENUM((dsModel), SensorStatusStatus, (none)(ok)(error));

DS_REFLECTED_ENUM((dsModel), ButtonType,
                  (none)(device)(area1)(area2)(area3)(area4)(zone)(zone1)(zone2)(zone3)(zone4)(zonex1)(zonex2)(zonex3)(
                      zonex4)(application)(group)(appmode));

DS_REFLECTED_ENUM((dsModel), ButtonMode, (none)(disabled)(button1way)(button2way));

DS_REFLECTED_ENUM(
    (dsModel), StateType,
    (none)(generic)(presence)(brightness)(presenceInDarkness)(twilight)(motion)(motionInDarkness)(smoke)(windMonitor)(
        rainMonitor)(sunRadiation)(roomThermostat)(batteryLow)(windowOpen)(doorOpen)(windowHandle)(garageDoorOpen)(
        sunProtection)(frost)(heatingSystemEnabled)(heatingChangeOver)(initialization)(malfunction)(service));

DS_REFLECTED_ENUM((dsModel), StateMode, (none)(disabled)(standard)(inverted));

DS_REFLECTED_ENUM((dsModel), StateStatusStatus, (none)(ok)(error));

DS_REFLECTED_ENUM(
    (dsModel), OutputType,
    (none)(lightBrightness)(lightHue)(lightSaturation)(lightTemperature)(lightCieX)(lightCieY)(shadePositionOutside)(
        shadePositionIndoor)(shadeOpeningAngleOutside)(shadeOpeningAngleIndoor)(shadeTransparency)(airFlowIntensity)(
        airFlowDirection)(airFlapOpeningAngle)(ventilationLouverPosition)(heatingPower)(coolingCapacity)(audioVolume)(
        powerState)(ventilationSwingMode)(ventilationAutoIntensity)(waterTemperature)(waterFlowRate));

DS_REFLECTED_ENUM((dsModel), OutputFunction, (none)(switched)(gradual)(positional)(internal));

DS_REFLECTED_ENUM((dsModel), OutputMode, (none)(disabled)(switched)(gradual)(positional)(internal));

DS_REFLECTED_ENUM((dsModel), OutputStatusStatus, (none)(ok)(moving)(dimming)(overload)(blocked)(error));

DS_REFLECTED_ENUM((dsModel), ZoneApplicationType,
                  (none)(lights)(shades)(audio)(video)(security)(access)(heating)(cooling)(temperature)(ventilation)(
                      recirculation)(window)(awnings)(joker));

DS_REFLECTED_ENUM((dsModel), AreaApplicationType, (none)(lights)(shades));

DS_REFLECTED_ENUM((dsModel), ClusterApplicationType, (none)(lights)(shades)(audio)(video)(heating)(cooling));

DS_REFLECTED_ENUM((dsModel), ShadeDeviceType, (none)(rollerShutter)(windowBlind)(awningTerrace)(awningFacade)(curtain));

DS_REFLECTED_ENUM(
    (dsModel), ShadeProtectionClass,
    (none)(undefined)(blinds1)(blinds2)(blinds3)(blinds4)(blinds5)(blinds6)(awnings1)(awnings2)(awnings3));

DS_REFLECTED_ENUM((dsModel), ShadeOrientation,
                  (none)(undefined)(north)(west)(south)(east)(northWest)(southWest)(northEast)(southEast));

// https://digitalstrom-zrh.atlassian.net/wiki/spaces/SA/pages/484671578/Standard+Scenarios (phase B without fireMuted)
DS_REFLECTED_ENUM((dsModel), ApartmentActionId,
                  (none)(absent)(present)(sleeping)(wakeup)(doorBell)(panic)(panicEnd)(fire)(fireEnd)(alarm1)(alarm2)(
                      alarm3)(alarm4)(alarm1End)(alarm2End)(alarm3End)(alarm4End));

DS_REFLECTED_ENUM((dsModel), LightsStatusStatus, (none)(off)(on)(dimming));
DS_REFLECTED_ENUM((dsModel), ShadesStatusStatus, (none)(closed)(open)(moving)(mixed));
DS_REFLECTED_ENUM((dsModel), AwningsStatusStatus, (none)(folded)(out)(moving)(mixed));
DS_REFLECTED_ENUM((dsModel), AudioStatusStatus, (none)(pause)(playing)(stop));
DS_REFLECTED_ENUM((dsModel), VentilationStatusStatus, (none)(off)(level1)(level2)(level3)(level4)(boost));
DS_REFLECTED_ENUM((dsModel), RecirculationStatusStatus, (none)(off)(level1)(level2)(level3)(level4)(boost));

DS_REFLECTED_ENUM((dsModel), UserDefinedStateStatusStatus, (none)(active)(inactive)(undefined));

DS_REFLECTED_ENUM((dsModel), ParameterCacheStatus, (none)(updating)(incomplete)(complete));

DS_REFLECTED_ENUM((dsModel), PowerKind, (none)(consumed)(produced)(selfConsumed));

DS_REFLECTED_ENUM((dsModel), TemperatureMode, (off)(automatic)(heating)(cooling));
DS_REFLECTED_ENUM(
    (dsModel), TemperatureLevel,
    (none)(heatingOff)(heatingComfort)(heatingEconomy)(heatingSetback)(heatingNight)(heatingVacation)(coolingOff)(
        coolingComfort)(coolingEconomy)(coolingSetback)(coolingNight)(coolingVacation)(passiveCoolingThreshold));

DS_REFLECTED_ENUM((dsModel), TemperatureStatusStatus, (none)(off)(heating)(cooling)(neutral));
DS_REFLECTED_ENUM((dsModel), TemperatureScene,
                  (none)(heatingOff)(heatingComfort)(heatingEconomy)(heatingSetback)(heatingNight)(heatingVacation)(
                      passiveCoolingActive)(passiveCoolingInactive)(coolingOff)(coolingComfort)(coolingEconomy)(
                      coolingSetback)(coolingNight)(coolingVacation));

DS_REFLECTED_ENUM((dsModel), Environment, (development)(alpha)(beta)(production));

namespace dsModel {

struct Origin {
  std::string type{};
  std::string id{};
};

struct SensorAttributes {
  std::string name{};
  std::string technicalName{};
  MeasurementType type{};
  MeasurementUsage usage{};
  std::optional<double> min{};
  std::optional<double> max{};
  std::optional<double> resolution{};
};

struct Sensor {
  std::string id{};
  SensorAttributes attributes{};
};

struct SensorStatus {
  std::string id{};
  std::optional<double> value{};
  SensorStatusStatus status{};
};

struct ButtonAttributes {
  std::string name{};
  std::string technicalName{};
  ButtonType type{};
  // hardwareSettings
  ButtonMode mode{};
  int channel{};
};

struct Button {
  std::string id{};
  ButtonAttributes attributes{};
};

struct StateAttributes {
  std::string name{};
  std::string technicalName{};
  StateType type{};
  // hardwareSettings
  StateMode mode{};
  std::optional<int> turnOnDelay{};
  std::optional<int> turnOffDelay{};
};

struct State {
  std::string id{};
  StateAttributes attributes{};
};

struct StateStatus {
  std::string id{};
  std::optional<double> value{};
  StateStatusStatus status{};
};

struct OutputAttributes {
  std::string name{};
  std::string technicalName{};
  OutputType type{};
  OutputFunction function{};
  std::optional<double> min{};
  std::optional<double> max{};
  std::optional<double> resolution{};
  // hardwareSettings
  OutputMode mode{};
  std::optional<int> levels;
  std::optional<std::vector<double>> levelThresholds;
};

struct Output {
  std::string id{};
  OutputAttributes attributes{};
};

struct OutputStatus {
  std::string id{};
  std::optional<double> value{};
  std::optional<double> initialValue{};
  OutputStatusStatus status{};
  std::optional<double> targetValue{}; // in case of slow progress
  ds::DateTime startedAt{};
  ds::DateTime terminatesAt{};         // predicted end time of process
  std::optional<int> level{};
};

struct FunctionBlockAttributes {
  std::string name{};
  std::string technicalName{};
  bool active{};
  // hardware modules
  std::vector<dsModel::Output> outputs{};
  std::vector<dsModel::Button> buttonInputs{};
  std::vector<dsModel::Sensor> sensorInputs{};
  // related
  std::string submodule{};
  std::string deviceAdapter{};
};
using FunctionBlock = dsApi::ResourceData<FunctionBlockId, FunctionBlockAttributes>;

struct FunctionBlockStatus {
  FunctionBlockId id{};
  std::vector<OutputStatus> outputs{};
  std::vector<SensorStatus> sensorInputs{};
};

struct SubmoduleAttributes {
  std::string name{};          // Wind sensor
  std::string technicalName{}; // GE-KM200
  // related
  std::string dsDevice{};                    // device dSUID
  std::vector<FunctionBlockId> functionBlocks{}; // List of function blocks dSUID
  std::string zone{};                        // Zone ID
  std::string application{};                 // Application, i.e. "joker"
};
using Submodule = dsApi::ResourceData<SubmoduleId, SubmoduleAttributes>;

struct ShadeAttributes {
  ShadeDeviceType shadeType{};
  ShadeProtectionClass protectionClass{};
  ShadeOrientation mountOrientation{};
};

struct DSDeviceAttributes {
  std::string name{}; // Kettle
  std::string model{};
  std::string gtin{};            // GTIN of the device
  std::string dsid{};            // DSID of the device
  std::string displayId{};       // displayId of the device
  std::optional<bool> present{}; // Device presence
  // registrationInfo
  ds::DateTime firstSeen{};
  ds::DateTime lastDiscovered{};
  ds::DateTime inactiveSince{};
  // shadeAttributes
  ShadeAttributes shadeAttributes{};
  // related
  std::vector<SubmoduleId> submodules{}; // List of device's submodules
  std::string zone{};                    // Id of zone where device is present
  std::vector<ScenarioId> scenarios{};
  std::vector<DeviceState> states{};
};
using DSDevice = dsApi::ResourceData<DeviceId, DSDeviceAttributes>;

struct DSDeviceStatusAttributes {
  std::vector<FunctionBlockStatus> functionBlocks{};
  std::vector<DeviceStateStatus> states{};
};
using DSDeviceStatus = dsApi::ResourceData<DeviceId, DSDeviceStatusAttributes>;

struct ZoneAttributes {
  std::string name{};              // Kitchen
  std::string floor{};             // Basement - Reference Id of the floor
  std::optional<double> orderId{}; // 20 - Order Id of the zone, used for sorting
  // related
  std::vector<SubmoduleId> submodules{};               // List of related Submodule Id's
  std::vector<ZoneApplicationType> applications{};     // List of Application Types in this zone (deprecated)
  std::vector<ZoneApplicationType> applicationTypes{}; // List of Application Types in this zone
  std::vector<ScenarioId> scenarios{};                 // List of Scenario Id's in this zone
};
using Zone = dsApi::ResourceData<ZoneId, ZoneAttributes>;

struct VentilationApplicationAttributes {
  ZoneApplicationType applicationType{};
}; // Not used yet

struct TemperatureSetpoint {
  TemperatureLevel id{};
  std::optional<double> value{};
};

struct TemperatureApplicationAttributes {
  ZoneApplicationType applicationType{};
  ZoneId zone{};
  std::vector<TemperatureLevel> availableLevels{};
  std::vector<TemperatureSetpoint> setpoints{};
};
using Application = dsApi::VariantResourceData<dsModel::ApplicationId, dsModel::TemperatureApplicationAttributes,
                                               dsModel::VentilationApplicationAttributes>;

struct ClusterAttributes {
  std::string name{};
  int orderId{};
  ClusterApplicationType applicationType{};
  std::vector<SubmoduleId> submodules{};
  std::string application{};
};
struct ClusterIncluded {
  std::optional<std::vector<Submodule>> submodules{};
  Application application{};
};
using Cluster = dsApi::ResourceDataWithIncluded<ClusterId, ClusterAttributes, ClusterIncluded>;

struct ClusterWeatherStatus {
  std::optional<bool> wind{};
  std::optional<bool> hail{};
};

struct ClusterStatusAttributes {
  ClusterWeatherStatus weather{};
};
using ClusterStatus = dsApi::ResourceData<ClusterId, ClusterStatusAttributes>;

struct ZoneMeasurementsStatus {
  std::optional<double> temperature{};
  std::optional<double> humidity{};
  std::optional<double> brightness{};
  std::optional<double> carbonDioxide{};
  std::optional<bool> motion{};
  std::optional<bool> presence{};
};

struct LightsStatus {
  LightsStatusStatus status{};
  LightsStatusStatus area1{}; // TODO(soon): add to documentation
  LightsStatusStatus area2{}; // TODO(soon): add to documentation
  LightsStatusStatus area3{}; // TODO(soon): add to documentation
  LightsStatusStatus area4{}; // TODO(soon): add to documentation
};

struct ShadesStatus {
  ShadesStatusStatus status{};
  ShadesStatusStatus area1{}; // TODO(soon): add to documentation
  ShadesStatusStatus area2{}; // TODO(soon): add to documentation
  ShadesStatusStatus area3{}; // TODO(soon): add to documentation
  ShadesStatusStatus area4{}; // TODO(soon): add to documentation
};

struct AwningsStatus {
  AwningsStatusStatus status{};
};

struct AudioStatus {
  AudioStatusStatus status{};
  std::optional<bool> muted{};
  std::optional<double> loudness{};
};

struct VentilationStatus {
  VentilationStatusStatus status{};
  std::optional<bool> malfunction{};
  std::optional<double> service{};
};

struct RecirculationStatus {
  RecirculationStatusStatus status{};
};

struct TemperatureStatus {
  std::optional<TemperatureMode> mode{};
  TemperatureStatusStatus status{};
  std::optional<bool> emergency{};
  std::optional<double> setpoint{};
  std::optional<double> controlValue{};
  TemperatureScene lastCalledScene{};
  std::optional<bool> setbackRequested{};
  std::optional<bool> ecoPreferred{};
};

// serialized to json as array by id
struct ZoneApplicationsStatus {
  LightsStatus lights{};
  ShadesStatus shades{};
  AwningsStatus awnings{};
  AudioStatus audio{};
  VentilationStatus ventilation{};
  RecirculationStatus recirculation{};
  TemperatureStatus temperature{};
};

struct ZoneStatusAttributes {
  ZoneMeasurementsStatus measurements{};
  ZoneApplicationsStatus applications{};   // List of Application Types in this zone
  std::vector<ScenarioStatus> scenarios{}; // List of Scenario Id's in this zone
};
using ZoneStatus = dsApi::ResourceData<ZoneId, ZoneStatusAttributes>;

struct FloorAttributes {
  std::string name{};
  std::optional<int> orderId{};
  std::vector<ZoneId> zones{};
};
using Floor = dsApi::ResourceData<FloorId, FloorAttributes>;
struct Floors {
  std::vector<Floor> floors{};
};

struct Coordinates {
  std::optional<double> longitude; // geographic coordinate in decimal degrees
  std::optional<double> latitude;  // geographic coordinate in decimal degrees
};

struct InstallationAttributes {
  Coordinates location;
  std::string countryCode; // ISO 3166-1 alpha-2 - country may be good to reserve for full country name
  std::string city;        // city in mtross words, location in dss property tree
};
using Installation = dsApi::ResourceData<InstallationId, InstallationAttributes>;

struct DSServerAttributes {
  std::string name{};
  std::string model{};
  std::string gtin{};
  std::string dsid{};
  std::string version{};
  std::optional<Environment> environment{};
};
using DSServer = dsApi::ResourceData<ServerId, DSServerAttributes>;

struct ApartmentAlarm {
  std::string name{};
};

struct ApartmentSecurity {
  ApartmentAlarm alarm1{};
  ApartmentAlarm alarm2{};
  ApartmentAlarm alarm3{};
  ApartmentAlarm alarm4{};
};

struct ApartmentAttributes {
  std::string name{};       // My digitalSTROM Home - Apartment name
  std::string deviceRoot{}; // `8794da01-40bf-4db1-ac1e-ab7ebabf8af5` - data model id of the attached DSS
  std::string createdAt{};  // `2010-05-23T00:38:10+02:00` - timestamp of apartment registration
  ApartmentSecurity security;

  // related
  std::string installation{};  // `034202d9-11d2-4885-af0a-a6b3d3c33f9e` - data model id of the related installation
  std::vector<ZoneId> zones{}; // List of Zone ID's
  std::vector<DeviceId> dsDevices{}; // List of dS-Device ID's
  std::vector<ClusterId> clusters{};
};
struct ApartmentIncluded {
  Installation installation;
  std::optional<std::vector<DSDevice>> dsDevices;
  std::optional<std::vector<Submodule>> submodules;
  std::optional<std::vector<FunctionBlock>> functionBlocks;
  std::optional<std::vector<Zone>> zones;
  std::optional<std::vector<Scenario>> scenarios;
  std::optional<std::vector<UserDefinedState>> userDefinedStates;
  std::optional<std::vector<Floor>> floors;
  std::optional<std::vector<Cluster>> clusters{};
  std::optional<std::vector<Application>> applications;
  std::optional<DSServer> dsServer;
};
using Apartment = dsApi::ResourceDataWithIncluded<std::string, ApartmentAttributes, ApartmentIncluded>;
extern const dsApi::EventType apartmentStructureChanged;

struct GroupStatus {
  std::string id{};
  // TODO(someday): refactor to std::variant<LightsStatus, ShadesStatus, AwningsStatus, AudioStatus, ...>
  ZoneApplicationsStatus application{}; // Only one application shall be set.
};

struct AppartmentApplicationsStatus {
  // TODO(someday)
  std::string dummy{};
};

struct ApartmentAccessStatus {
  std::optional<bool> absent{};
};

struct ApartmentUserStatus {
  std::optional<bool> sleeping{};
  std::optional<bool> holiday{};
};

struct ApartmentSecurityStatus {
  std::optional<bool> panic{};
  std::optional<bool> fire{};
  std::optional<bool> smoke{};
  std::optional<bool> gas{};
  std::optional<bool> pollution{};
  std::optional<bool> alarm1{};
  std::optional<bool> alarm2{};
  std::optional<bool> alarm3{};
  std::optional<bool> alarm4{};
  std::optional<bool> burglaryAlarm{};
  std::optional<bool> burglaryAttempt{};
};

struct ApartmentTemperatureStatus {
  std::optional<TemperatureMode> mode{};
  std::optional<bool> ecoPreferred{};
};

struct ApartmentWeatherStatus {
  std::optional<bool> wind{};
  std::optional<bool> rain{};
  std::optional<bool> hail{};
  std::optional<bool> frost{};
};

struct ApartmentEnvironmentStatus {
  std::optional<bool> daylight{};
  std::optional<bool> twilight{};
  std::optional<bool> daynight{};
  ds::DateTime sunrise{};
  ds::DateTime sunset{};
};

struct ApartmentMeasurementStatus {
  std::optional<double> temperature{};   // Celsius degree
  std::optional<double> humidity{};      // %rH
  std::optional<double> brightness{};    // lux
  std::optional<double> airPressure{};   // hPa
  std::optional<double> precipitation{}; // mm in last hour
  std::optional<double> windSpeed{};     // m/s
  std::optional<double> windGust{};      // m/s
};

struct ApartmentStatusAttributes {
  AppartmentApplicationsStatus applications{};
  ApartmentAccessStatus access{};
  ApartmentUserStatus user{};
  ApartmentSecurityStatus security{};
  ApartmentTemperatureStatus temperature{};
  ApartmentWeatherStatus weather{};
  ApartmentEnvironmentStatus environment{};
  ApartmentMeasurementStatus measurements{};
};
struct ApartmentStatusIncluded {
  std::optional<std::vector<DSDeviceStatus>> dsDevices;
  std::optional<std::vector<ZoneStatus>> zones;
  std::optional<std::vector<ClusterStatus>> clusters;
  std::optional<std::vector<UserDefinedStateStatus>> userDefinedStates;
};
using ApartmentStatus =
    dsApi::ResourceDataWithIncluded<std::string, ApartmentStatusAttributes, ApartmentStatusIncluded>;
extern const dsApi::EventType apartmentStatusChanged;

struct FunctionBlockParameterCacheAttributes {
  std::optional<double> completion{};
};
using FunctionBlockParameterCache = dsApi::ResourceData<std::string, FunctionBlockParameterCacheAttributes>;

struct MeteringAttributes {
  std::string unit{};
  std::string technicalName{};
  Origin origin{};
};
using Metering = dsApi::ResourceData<MeteringId, MeteringAttributes>;
struct Meterings {
  std::vector<Metering> meterings{};
};

struct MeteringValueAttributes {
  std::optional<double> value;
};
using MeteringValue = dsApi::ResourceData<MeteringId, MeteringValueAttributes>;
struct MeteringValues {
  std::vector<MeteringValue> values{};
};
extern const dsApi::EventType meteringValues;

struct ParameterCacheAttributes {
  ParameterCacheStatus status;
  std::optional<int> total{};
  std::optional<int> completed{};
  std::optional<double> completion{};
  std::optional<ds::Clock::duration> remainingTime{};
};
struct ParameterCacheIncluded {
  std::optional<std::vector<dsModel::FunctionBlockParameterCache>> functionBlocks{};
};
using ParameterCache = dsApi::ResourceDataWithIncluded<std::string, ParameterCacheAttributes, ParameterCacheIncluded>;

struct UserDefinedStateAttributes {
  std::string name{};
  std::string setName{};
  std::string resetName{};
  std::optional<bool> visibleForUsers{};
};
using UserDefinedState = dsApi::ResourceData<std::string, UserDefinedStateAttributes>;

struct UserDefinedStateStatusAttributes {
  UserDefinedStateStatusStatus status{};
};
using UserDefinedStateStatus = dsApi::ResourceData<std::string, UserDefinedStateStatusAttributes>;

struct DeviceState {
  DeviceStateId id{};
  std::vector<DeviceStateOption> options{};
  dsApi::Text name{};
};

struct DeviceStateStatus {
  DeviceStateId id{};
  std::string value{};
};

struct DeviceStateOption {
  std::string id{};
  dsApi::Text name{};
};

} // namespace dsModel

DS_JSON_DECLARE_STRUCT((dsModel), Origin, (type)(id));

DS_JSON_DECLARE_STRUCT((dsModel), SensorAttributes, (name)(technicalName)(type)(usage)(min)(max)(resolution));
DS_JSON_DECLARE_STRUCT((dsModel), Sensor, (id)(attributes));
DS_JSON_DECLARE_STRUCT((dsModel), SensorStatus, (id)(value)(status));

DS_JSON_DECLARE_STRUCT((dsModel), ButtonAttributes, (name)(technicalName)(type)(mode)(channel));
DS_JSON_DECLARE_STRUCT((dsModel), Button, (id)(attributes));

DS_JSON_DECLARE_STRUCT((dsModel), StateAttributes, (name)(technicalName)(type)(mode)(turnOnDelay)(turnOffDelay));
DS_JSON_DECLARE_STRUCT((dsModel), State, (id)(attributes));
DS_JSON_DECLARE_STRUCT((dsModel), StateStatus, (id)(value)(status));

DS_JSON_DECLARE_STRUCT((dsModel), OutputAttributes,
                       (name)(technicalName)(type)(function)(min)(max)(resolution)(mode)(levels)(levelThresholds));
DS_JSON_DECLARE_STRUCT((dsModel), Output, (id)(attributes));
DS_JSON_DECLARE_STRUCT((dsModel), OutputStatus, (id)(value)(initialValue)(status)(targetValue)(startedAt)(terminatesAt)(level));

DS_JSON_DECLARE_STRUCT(
    (dsModel), FunctionBlockAttributes,
    (name)(technicalName)(active)(outputs)(buttonInputs)(sensorInputs)(submodule)(deviceAdapter));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), FunctionBlock);
DS_JSON_DECLARE_STRUCT((dsModel), FunctionBlockStatus, (id)(outputs)(sensorInputs));

DS_JSON_DECLARE_STRUCT((dsModel), SubmoduleAttributes,
                       (name)(technicalName)(dsDevice)(functionBlocks)(zone)(application));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), Submodule);

DS_JSON_DECLARE_STRUCT((dsModel), ShadeAttributes, (shadeType)(protectionClass)(mountOrientation));

DS_JSON_DECLARE_STRUCT((dsModel), DSDeviceAttributes,
                       (name)(model)(gtin)(dsid)(displayId)(present)(firstSeen)(lastDiscovered)(inactiveSince)(
                           shadeAttributes)(submodules)(zone)(scenarios)(states));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), DSDevice);
DS_JSON_DECLARE_STRUCT((dsModel), DSDeviceStatusAttributes, (functionBlocks)(states));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), DSDeviceStatus);

DS_JSON_DECLARE_STRUCT((dsModel), ZoneAttributes,
                       (name)(floor)(orderId)(submodules)(applications)(applicationTypes)(scenarios));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), Zone);

DS_JSON_DECLARE_STRUCT((dsModel), ClusterAttributes, (name)(orderId)(applicationType)(submodules)(application));
DS_JSON_DECLARE_STRUCT((dsModel), ClusterIncluded, (submodules)(application));
DS_JSON_DECLARE_API_RESOURCE_DATA_WITH_INCLUDED((dsModel), Cluster);
DS_JSON_DECLARE_STRUCT((dsModel), ClusterWeatherStatus, (wind)(hail));
DS_JSON_DECLARE_STRUCT((dsModel), ClusterStatusAttributes, (weather));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), ClusterStatus);

DS_JSON_DECLARE_STRUCT((dsModel), VentilationApplicationAttributes, (applicationType));
DS_JSON_DECLARE_STRUCT((dsModel), TemperatureSetpoint, (id)(value));
DS_JSON_DECLARE_STRUCT((dsModel), TemperatureApplicationAttributes,
                       (applicationType)(zone)(availableLevels)(setpoints));
static constexpr auto dsModelApplicationNameVisitor = ::ds::Visitor{
    [](const std::monostate&) { return ""; },
    [](const dsModel::TemperatureApplicationAttributes&) { return "temperatureApplication"; },
    [](const dsModel::VentilationApplicationAttributes&) { return "ventilationApplication"; },
};
DS_JSON_DECLARE_API_VARIANT_RESOURCE_DATA((dsModel), Application, dsModelApplicationNameVisitor);

DS_JSON_DECLARE_STRUCT((dsModel), Coordinates, (longitude)(latitude));
DS_JSON_DECLARE_STRUCT((dsModel), InstallationAttributes, (location)(countryCode)(city));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), Installation);

DS_JSON_DECLARE_STRUCT((dsModel), DSServerAttributes, (name)(model)(gtin)(dsid)(version)(environment));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), DSServer);

DS_JSON_DECLARE_STRUCT((dsModel), ApartmentAlarm, (name));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentSecurity, (alarm1)(alarm2)(alarm3)(alarm4));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentAttributes, (name)(deviceRoot)(createdAt)(security)(installation)(zones)(dsDevices)(clusters));
DS_JSON_DECLARE_STRUCT(
    (dsModel), ApartmentIncluded,
    (installation)(dsDevices)(submodules)(functionBlocks)(zones)(scenarios)(userDefinedStates)(floors)(clusters)(applications)(dsServer));
DS_JSON_DECLARE_API_RESOURCE_DATA_WITH_INCLUDED((dsModel), Apartment);

DS_JSON_DECLARE_STRUCT((dsModel), ZoneMeasurementsStatus,
                       (temperature)(humidity)(brightness)(carbonDioxide)(motion)(presence));
DS_JSON_DECLARE_STRUCT((dsModel), LightsStatus, (status)(area1)(area2)(area3)(area4));
DS_JSON_DECLARE_STRUCT((dsModel), ShadesStatus, (status)(area1)(area2)(area3)(area4));
DS_JSON_DECLARE_STRUCT((dsModel), AwningsStatus, (status));
DS_JSON_DECLARE_STRUCT((dsModel), AudioStatus, (status)(muted)(loudness));
DS_JSON_DECLARE_STRUCT((dsModel), VentilationStatus, (status)(malfunction)(service));
DS_JSON_DECLARE_STRUCT((dsModel), RecirculationStatus, (status));
DS_JSON_DECLARE_STRUCT(
    (dsModel), TemperatureStatus,
    (mode)(status)(emergency)(setpoint)(controlValue)(lastCalledScene)(setbackRequested)(ecoPreferred))
template <>
struct DSJsonArrayId<dsModel::ZoneApplicationsStatus> : public std::true_type {};
DS_JSON_DECLARE_STRUCT((dsModel), ZoneApplicationsStatus,
                       (lights)(shades)(awnings)(audio)(ventilation)(recirculation)(temperature));
DS_JSON_DECLARE_STRUCT((dsModel), ZoneStatusAttributes, (measurements)(applications)(scenarios));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), ZoneStatus);
DS_JSON_DECLARE_STRUCT((dsModel), GroupStatus, (id)(application));
DS_JSON_DECLARE_STRUCT((dsModel), AppartmentApplicationsStatus, (dummy));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentAccessStatus, (absent));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentUserStatus, (sleeping)(holiday));
DS_JSON_DECLARE_STRUCT(
    (dsModel), ApartmentSecurityStatus,
    (panic)(fire)(smoke)(gas)(pollution)(alarm1)(alarm2)(alarm3)(alarm4)(burglaryAlarm)(burglaryAttempt));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentTemperatureStatus, (mode)(ecoPreferred));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentWeatherStatus, (wind)(rain)(hail)(frost));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentEnvironmentStatus, (daylight)(twilight)(daynight)(sunrise)(sunset));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentMeasurementStatus,
                       (temperature)(humidity)(brightness)(airPressure)(precipitation)(windSpeed)(windGust));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentStatusAttributes,
                       (applications)(access)(user)(security)(temperature)(weather)(environment)(measurements));
DS_JSON_DECLARE_STRUCT((dsModel), ApartmentStatusIncluded, (dsDevices)(zones)(clusters)(userDefinedStates));
DS_JSON_DECLARE_API_RESOURCE_DATA_WITH_INCLUDED((dsModel), ApartmentStatus);
DS_JSON_DECLARE_STRUCT((dsModel), FunctionBlockParameterCacheAttributes, (completion));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), FunctionBlockParameterCache);
DS_JSON_DECLARE_STRUCT((dsModel), ParameterCacheAttributes, (status)(total)(completed)(completion)(remainingTime));
DS_JSON_DECLARE_STRUCT((dsModel), ParameterCacheIncluded, (functionBlocks));
DS_JSON_DECLARE_API_RESOURCE_DATA_WITH_INCLUDED((dsModel), ParameterCache);
DS_JSON_DECLARE_STRUCT((dsModel), UserDefinedStateAttributes, (name)(setName)(resetName)(visibleForUsers));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), UserDefinedState);
DS_JSON_DECLARE_STRUCT((dsModel), UserDefinedStateStatusAttributes, (status));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), UserDefinedStateStatus);

DS_JSON_DECLARE_STRUCT((dsModel), MeteringAttributes, (unit)(technicalName)(origin));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), Metering);
DS_JSON_DECLARE_STRUCT((dsModel), Meterings, (meterings));

DS_JSON_DECLARE_STRUCT((dsModel), MeteringValueAttributes, (value));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), MeteringValue);
DS_JSON_DECLARE_STRUCT((dsModel), MeteringValues, (values));

DS_JSON_DECLARE_STRUCT((dsModel), FloorAttributes, (name)(orderId)(zones));
DS_JSON_DECLARE_API_RESOURCE_DATA((dsModel), Floor);
DS_JSON_DECLARE_STRUCT((dsModel), Floors, (floors));

DS_JSON_DECLARE_STRUCT((dsModel), DeviceState, (id)(options)(name));
DS_JSON_DECLARE_STRUCT((dsModel), DeviceStateStatus, (id)(value));
DS_JSON_DECLARE_STRUCT((dsModel), DeviceStateOption, (id)(name));
