#include "scenarios.h"

#include <ds-api/service.h>
#include "model.h"

DS_JSON_DEFINE_STRUCT((dsModel), UserDefinedActionAttributes);
DS_JSON_DEFINE_STRUCT((dsModel), ApplicationZoneScenarioAttributes);
DS_JSON_DEFINE_STRUCT((dsModel), DeviceScenarioAttributes);
DS_JSON_DEFINE_STRUCT((dsModel), Scenario);
DS_JSON_DEFINE_STRUCT((dsModel), InvokeStandardScenarioArgs);
DS_JSON_DEFINE_STRUCT((dsModel), ScenarioArgs);

namespace dsModel {

std::vector<dsApi::Route> apartmentApiRoutes(ScenariosApi& api) {
  return {
      // must be before "/scenarios/{scenarioId}"
      dsApi::makeMethodRoute<InvokeStandardScenarioArgs>(
          "/scenarios/invoke", [&api](auto&& args) { api.invokeStandardScenario(std::forward<decltype(args)>(args)); }),
      dsApi::makeDocumentRoute("/scenarios", {.methods = {dsApi::makeCreate<Scenario>([&api](auto&& x) {
                                                return api.createScenario(std::forward<decltype(x)>(x));
                                              })}}),
      dsApi::makeDocumentRoute(
          "/scenarios/{scenarioId}",
          {.methods = {dsApi::makeDelete<ScenarioArgs>(
               [&api](auto&& x) { return api.deleteScenario(std::forward<decltype(x)>(x)); })},
           .patchRoutes = {{.pattern = "/name",
                            .operations = {dsApi::makeReplacePatch<dsModel::ScenarioArgs, std::string>(
                                [&api](auto&& args, auto&& value) {
                                  api.setScenarioName(std::forward<decltype(args)>(args),
                                                      std::forward<decltype(value)>(value));
                                })}}}}),
      dsApi::makeMethodRoute<ScenarioArgs>(
          "/scenarios/{scenarioId}/invoke",
          [&api](auto&& args) { api.invokeScenario(std::forward<decltype(args)>(args)); }),
      dsApi::makeMethodRoute<ScenarioArgs>(
          "/scenarios/{scenarioId}/persist",
          [&api](auto&& args) { api.persistScenario(std::forward<decltype(args)>(args)); }),
  };
}

} // namespace dsModel
