#include "reconnect-timer.h"

using namespace std::literals;

namespace ds::asio {

ReconnectTimer::ReconnectTimer(boost::asio::io_service& io) : m_timer{io} {}

ReconnectTimer::~ReconnectTimer() {}

void ReconnectTimer::connected() {
  m_connectedAt = Timer::clock_type::now();
}

void ReconnectTimer::resetSchedule() {
  m_timer.cancel();
  m_scheduleIt = m_schedule.cbegin();
}

Timer::duration ReconnectTimer::nextSchedule() {
  if (m_connectedAt) {
    if (Timer::clock_type::now() - *m_connectedAt > m_resetScheduleAfterConnectedDuration) {
      // connection was up long enough to reset the schedule
      resetSchedule();
    }
    m_connectedAt = {};
  }

  auto result = *m_scheduleIt;
  if (std::next(m_scheduleIt) != m_schedule.cend()) {
    ++m_scheduleIt;
  }
  return result;
}

} // namespace ds::asio
