/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "resource.h"
#include <ds/asio/test.h>

namespace {

struct Config {
  std::string s;
};
} // namespace

DS_REFLECT_ANONYMOUS_STRUCT(Config, (s));

namespace {

static const char* TAGS = "[dsAsioResource][dsAsio][ds]";
using namespace std::chrono_literals;
using namespace trompeloeil;

TEMPLATE_TEST_CASE("dsAsioResource", TAGS, ds::asio::NoResourceConfig, Config) {
  auto loop = ds::asio::TestLoopResource{};

  WHEN("Created from context") {
    auto resource = ds::asio::Resource<TestType>(loop.context, "resource");
    THEN("context, id, io are set") {
      CHECK(&resource.context() == &loop.context);
      CHECK(resource.id() == "resource");
      CHECK(&resource.io() == &loop.io);
    }

    WHEN("child resource") {
      auto childResource = ds::asio::Resource<>(resource, "child");
      THEN("context, id, io are set") {
        CHECK(&childResource.context() == &loop.context);
        CHECK(childResource.id() == "resource-child");
        CHECK(&childResource.io() == &loop.io);
      }
    }
  }
}

TEST_CASE("dsAsioResource with config without conf entry", TAGS) {
  auto loop = ds::asio::TestLoopResource{};
  WHEN("Created from context") {
    auto defaultConfig = Config{.s = "default"};
    auto resource = ds::asio::Resource<Config>(loop.context, "resource", Config{defaultConfig});
    THEN("config matches defaultConfig") { CHECK(resource.config() == defaultConfig); }

    WHEN("child resource") {
      auto defaultChildConfig = Config{.s = "childDefault"};
      auto childResource = ds::asio::Resource<Config>(resource, "child", Config{defaultChildConfig});
      THEN("config matches defaultConfig") { CHECK(childResource.config() == defaultChildConfig); }
    }
  }
}

TEST_CASE("dsAsioResource with config with conf entry", TAGS) {
  auto loop = ds::asio::TestLoopResource{};

  WHEN("Created from context") {
    const auto config = Config{.s = "X"};
    const auto defaultConfig = Config{.s = "default"};
    REQUIRE_CALL(loop.system.conf(), readJsonIfExists(eq("resource"))).RETURN(ds::toJsonFragment(config));
    auto resource = ds::asio::Resource<Config>(loop.context, "resource", Config{defaultConfig});
    THEN("config matches conf") { CHECK(resource.config() == config); }

    WHEN("child resource") {
      const auto childConfig = Config{.s = "childX"};
      const auto defaultChildConfig = Config{.s = "childDefault"};
      REQUIRE_CALL(loop.system.conf(), readJsonIfExists(eq("resource-child"))).RETURN(ds::toJsonFragment(childConfig));

      auto childResource = ds::asio::Resource<Config>(resource, "child", Config{defaultChildConfig});
      THEN("config matches conf") { CHECK(childResource.config() == childConfig); }
    }
  }
}

} // namespace
