/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "retryer.h"
#include <ds/asio/back-off.h>
#include <ds/asio/timer.h>
#include <ds/log.h>

static ds::LogChannel dsLogChannel("dsAsioRetryer");

namespace ds::asio {

using namespace std::literals;

void Retryer::asyncPerform(RetryerHooks&& hooks) {
  DS_INFO_ENTER(*this);
  m_backOff.reset();
  asyncRetryLoop(std::move(hooks));
}

void Retryer::asyncRetryLoop(RetryerHooks&& hooks) {
  DS_DEBUG_ENTER(*this);
  m_timer.randomlyExpiresFromNow(m_backOff.next(), [this, hooks = std::move(hooks)]() mutable {
    DS_CONTEXT(*this);
    DS_INFO_ENTER("");
    try {
      hooks.attempt();
      DS_INFO("Attemp succesful");
      if (hooks.succeeded) {
        hooks.succeeded();
      }
    } catch (const std::exception& e) {
      DS_WARNING("Attempt failed.", e);
      if (hooks.attemptFailed) {
        hooks.attemptFailed();
      }
      asyncRetryLoop(std::move(hooks));
    }
  });
}

} // namespace ds::asio
