/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include <ds/catch/catch.h>
#include <ds/clock-mock.h>
#include <ds/system-mock.h>
#include <boost/asio/io_service.hpp>
#include "resource.h"
#include "timer.h"

namespace ds::asio {
using namespace std::literals;

struct TestLoop {
  boost::asio::io_service io{};

  TestLoop() { asyncLoop(); }

  void stop() { io.stop(); }

  // runs and stops
  void run() {
    io.reset();
    io.run();
  };

  void postStop() {
    io.post([&] { io.stop(); });
  };

  void postStopRun() {
    postStop();
    run();
  };

  void postPostStop() {
    io.post([&] { io.post([&] { io.stop(); }); });
  };

  void postPostStopRun() {
    postPostStop();
    run();
  };

  // runs for roughly x and stops
  void runFor(ds::ClockMocker& clock, ds::Clock::duration x) {
    clock.advance(x * 4 / 5);
    postStopRun();
    clock.advance(x * 1 / 5);
    run();
  }

private:
  // Timer with asynch loop as work.
  // Detects stalled loop in combination with ClockMocker.
  // It leads to failed test instead of stalled test.
  ds::asio::Timer m_timer{io};

  void asyncLoop() {
    m_timer.randomlyExpiresFromNowPercentDown(1h, 10, [this] { asyncLoop(); });
  }
};

struct TestLoopResource : public TestLoop {
  ds::SystemMock system;
  ResourceContext context{.io = io, .system = system};
  ds::asio::Resource<> resource{context, "test"};
};

} // namespace ds::asio
