/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "http.h"
#include <ds/catch/catch.h>

namespace {

static const char *TAGS = "[dsHttp][ds]";

TEST_CASE("dsUrlDecode", TAGS) {
  SECTION("% sequences are decoded") {
    CHECK(ds::urlDecode("%20") == " ");
    CHECK(ds::urlDecode("a%20") == "a ");
    CHECK(ds::urlDecode("%20a") == " a");
    CHECK(ds::urlDecode("v%20a") == "v a");
    CHECK(ds::urlDecode("%20%20") == "  ");
    CHECK(ds::urlDecode("%20%20%20") == "   ");
  }

  SECTION("Invalid % sequences behave sane") {
    CHECK(ds::urlDecode("%") == "");
    CHECK(ds::urlDecode("%4") == "");
    CHECK(ds::urlDecode("%4z") == "@");
    CHECK(ds::urlDecode("%z9") == "\t");
  }

  SECTION("+ is decoded as space") {
    CHECK(ds::urlDecode("+") == " ");
    CHECK(ds::urlDecode("++") == "  ");
    CHECK(ds::urlDecode("+a+") == " a ");
    CHECK(ds::urlDecode("++a") == "  a");
    CHECK(ds::urlDecode("a++") == "a  ");
    CHECK(ds::urlDecode("+%20b") == "  b");
    CHECK(ds::urlDecode("+%41b") == " Ab");
  }

  SECTION("complex case") {
    CHECK(ds::urlDecode("sourceid=1&schedule=FREQ%3DMINUTELY%3BINTERVAL%3D1&start=20080520T080000Z") ==
          "sourceid=1&schedule=FREQ=MINUTELY;INTERVAL=1&start=20080520T080000Z");
  }
}

TEST_CASE("dsTryHttpQueryArg", TAGS) {
  CHECK(ds::tryHttpQueryArg("", "aa") == std::nullopt);
  CHECK(ds::tryHttpQueryArg("aa=AA", "aa").value() == "AA");
  CHECK(ds::tryHttpQueryArg("aa=AA&bb=BB", "aa").value() == "AA");
  CHECK(ds::tryHttpQueryArg("aa=AA&bb=BB", "bb").value() == "BB");
  CHECK(ds::tryHttpQueryArg("aa=AA&bb=BB", "ac") == std::nullopt);

  CHECK(ds::tryHttpQueryArg("aa=", "aa").value() == "");
  CHECK(ds::tryHttpQueryArg("aa=&bb=BB", "aa").value() == "");
  CHECK(ds::tryHttpQueryArg("aa=AA&bb=", "bb").value() == "");

  CHECK(ds::tryHttpQueryArg("aa=A+%41&bb=BB", "aa").value() == "A A");
}

TEST_CASE("dsHttpQueryArg", TAGS) {
  CHECK(ds::httpQueryArg("aa=AA&bb=BB", "aa") == "AA");
  CHECK_THROWS_FIND(ds::httpQueryArg("aa=AA&bb=BB", "ab"), "Http query argument not found. name:ab");
}
} // namespace
