/*
    Copyright (c) 2018 digitalSTROM AG, Zurich, Switzerland

    This file is part of dSS.

    vdSM is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2 of the License.

    vdSM is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with dSS. If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/filesystem/operations.hpp>

#include "ds/catch/catch.h"
#include "ds/sqlite3/ds-sqlite3.h"

static const std::string TEST_STATIC_DATADIR(DS_SRC_DIR "test/");

static std::string TAGS{"[dss][dsSqlite3]"};

namespace {

SCENARIO("Corrupted database Writing", TAGS + "[.][sqlite3CorruptWriting]") {
  boost::filesystem::copy_file(TEST_STATIC_DATADIR + "cache.corrupted.db.orig", "cache.corrupted.db",
                               boost::filesystem::copy_option::overwrite_if_exists);
  GIVEN("a corrupt database") {
    ds::sqlite3::Database d("cache.corrupted.db");
    WHEN("Writing an entry") {
      d.exec("INSERT OR REPLACE INTO TestEntry_table (dsuid, data) VALUES(?, ?);", "0000000000000000000000000000000001", R"({"a":42,"b":"device-caching ?/"})");
      THEN("Program aborts") {
        REQUIRE(false); // Must not be reached, abort before.
      }
    }
  }
  REQUIRE(boost::filesystem::is_regular_file("cache.corrupted.corrupted.db"));
  boost::filesystem::remove("cache.corrupted.corrupted.db");
}

SCENARIO("Corrupted database Reading", TAGS + "[.][sqlite3CorruptReading]") {
  boost::filesystem::copy_file(TEST_STATIC_DATADIR + "cache.corrupted.db.orig", "cache.corrupted.db",
                               boost::filesystem::copy_option::overwrite_if_exists);
  ds::sqlite3::Database d("cache.corrupted.db");
  GIVEN("a corrupt database") {
    WHEN("Reading an entry") {
      std::string entryStr;
      std::tie(entryStr) =
          d.exec("SELECT data FROM TestEntry_table WHERE dsuid = ?;", "0000000000000000000000000000000001").template row<std::string>();
      THEN("Program aborts") {
        REQUIRE(false); // Must not be reached, abort before.
      }
    }
  }
  REQUIRE(boost::filesystem::is_regular_file("cache.corrupted.corrupted.db"));
  boost::filesystem::remove("cache.corrupted.corrupted.db");
}

SCENARIO("Corrupted database Removing", TAGS + "[.][sqlite3CorruptRemoving]") {
  boost::filesystem::copy_file(TEST_STATIC_DATADIR + "cache.corrupted.db.orig", "cache.corrupted.db",
                               boost::filesystem::copy_option::overwrite_if_exists);
  ds::sqlite3::Database d("cache.corrupted.db");
  GIVEN("a corrupt database") {
    WHEN("Removing an entry") {
      d.exec("DELETE FROM TestEntry_table WHERE dsuid = ?;", "0000000000000000000000000000000001");
      THEN("Program aborts") {
        REQUIRE(false); // Must not be reached, abort before.
      }
    }
  }
  REQUIRE(boost::filesystem::is_regular_file("cache.corrupted.corrupted.db"));
  boost::filesystem::remove("cache.corrupted.corrupted.db");
}

} // namespace
