/*
    Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "str.h"
#include <cctype>
#include <locale>
#include "ds/log.h"
namespace ds {

std::optional<int> tryToInt(const std::string& s) {
  if (s.empty() || isspace(s[0]) || (s[0] == '0' && s[1] != 0)) {
    return {};
  }
  char* endp;
  auto result = strtol(s.c_str(), &endp, 10);
  if (endp == (s.data() + s.size())) {
    return result;
  }
  return {};
}

int requireInt(const std::string& s) {
  if (auto x = tryToInt(s)) {
    return *x;
  }
  DS_FAIL_REQUIRE("Conversion to integer failed.", s);
}

std::optional<double> tryToDouble(const std::string& s) {
  if (s.empty() || isspace(s[0]) || (s[0] == '0' && s[1] != 0)) {
    return {};
  }
  char* endp;
  auto result = strtod(s.c_str(), &endp);
  if (endp == s.data() + s.size()) {
    return result;
  }
  return {};
}

double requireDouble(const std::string& s) {
  if (auto x = tryToDouble(s)) {
    return *x;
  }
  DS_FAIL_REQUIRE("Conversion to double failed.", s);
}

std::string intToString(const long long integer, int base) {
  //
  // http://en.cppreference.com/w/cpp/types/numeric_limits/digits10
  // ... any number with this many decimal digits can be converted to a value
  // of type T and back to decimal form, without change due to rounding or
  // overflow.
  // It's a lower bound, the type may also hold numbers with one digit more
  // but not all of them
  // +3 for sign, '\0' terminator and upper bound
  //
  const int max_size = std::numeric_limits<long long>::digits10 + 3;
  char buffer[max_size] = { 0 };
  int n;
  if (base == 16) {
    n = snprintf(buffer, max_size, "0x%llx", integer);
  } else {
    n = snprintf(buffer, max_size, "%lld", integer);
  }
  DS_REQUIRE(n < max_size, n, max_size);
  return std::string(buffer);
}

namespace detail {

void Str<unsigned char>::f(std::ostream& ostream, unsigned char x) {
  ostream << static_cast<int>(x);
}
void Str<signed char>::f(std::ostream& ostream, signed char x) {
  ostream << static_cast<int>(x);
}
void Str<std::exception>::f(std::ostream& ostream, const std::exception& x) {
  ostream << x.what();
}

} // namespace detail
} // namespace ds
