/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <trompeloeil.hpp>
#include "conf-mock.h"
#include "system.h"

namespace ds {

using namespace std::literals;
using namespace trompeloeil;

struct SystemMock : public trompeloeil::mock_interface<System> {
public:
  IMPLEMENT_CONST_MOCK0(dirs);
  IMPLEMENT_CONST_MOCK1(readFile);
  IMPLEMENT_CONST_MOCK1(readFileIfExists);
  IMPLEMENT_CONST_MOCK0(now);
  ds::Conf& conf() const override { return m_conf; }
  ds::ConfMock& conf() { return m_conf; }
  IMPLEMENT_CONST_MOCK0(release);
  IMPLEMENT_CONST_MOCK0(isInternalRelease);

  // Mock system clock to 2017-09-18T10:37:00.777Z by default
  static constexpr ds::DateTime defaultNow{1505731033777ms};

private:
  mutable ConfMock m_conf;
  std::unique_ptr<trompeloeil::expectation> m_allowNow{NAMED_ALLOW_CALL(*this, now()).RETURN(defaultNow)};
  std::unique_ptr<trompeloeil::expectation> m_allowRelease{NAMED_ALLOW_CALL(*this, release()).RETURN(Release::devel)};
};

} // namespace ds
