/*
    Copyright (c) 2018 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include "system.h"
#include <ds/catch/catch.h>
#include <ds/log.h>

static const char* TAGS = "[dsSystem][ds]";

namespace {

TEST_CASE("dsSystem", TAGS) {
  SECTION("dirs from build system by default") {
    auto& system = ds::system();
    const auto& dirs = system.dirs();
    CHECK(dirs.sysconfdir == DS_SYSCONFDIR);
    CHECK(dirs.datadir == DS_DATADIR);
  }

  SECTION("dirs passed in constructor") {
    auto system = ds::makeSystem({.sysconfdir = "/sysconfdir", .datadir = "/datadir"});
    const auto& dirs = system->dirs();
    CHECK(dirs.sysconfdir == "/sysconfdir");
    CHECK(dirs.datadir == "/datadir");
  }

  SECTION("testSystem") {
    // DS_SRC_DIR is passed as -D... by build system
    const auto& path = DS_SRC_DIR "/test";
    auto& testSystem = ds::system();

    SECTION("readFile reads existing file") { CHECK(testSystem.readFile(ds::str(path, "/true-line")) == "true\n"); }

    SECTION("readFile throws on non-existing file") {
      CHECK_THROWS(testSystem.readFile(ds::str(path, "/non-existing")));
    }

    SECTION("readFileIfExists reads existing file") {
      CHECK(testSystem.readFile(ds::str(path, "/true-line")) == "true\n");
    }

    SECTION("readFileIfExists returns none on non-existing file") {
      CHECK(testSystem.readFileIfExists(ds::str(path, "non-existing")) == std::nullopt);
    }
  }
  SECTION("now returns system_clock") {
    auto& system = ds::system();
    auto dsNow = system.now();
    auto stdNow = std::chrono::system_clock::now();
    CHECK(dsNow <= stdNow);
    CHECK(stdNow - dsNow <= std::chrono::seconds(5));
  }
}

} // namespace
