/*
 *  Copyright (c) 2019 digitalSTROM AG, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <trompeloeil.hpp>
#include "hal/hardware-abstraction.h"

namespace dss {

class MockHal : public HardwareAbstraction {
public:
  explicit MockHal(Apartment* apt, ds::sqlite3::Database* db) : HardwareAbstraction(apt, db) {}
  MAKE_CONST_MOCK2(readDouble, double(const Dsuid& dsuid, const std::string& parameter), override);
  MAKE_CONST_MOCK2(readString, std::string(const Dsuid& dsuid, const std::string& parameter), override);
  MAKE_CONST_MOCK2(readBool, bool(const Dsuid& dsuid, const std::string& parameter), override);
  MAKE_CONST_MOCK4(callScene, bool(const Dsuid& dsuid, int sceneId, bool forced, ds::DateTime now), override);
  MAKE_CONST_MOCK3(undoScene, bool(const Dsuid& dsuid, int sceneId, ds::DateTime now), override);
  MAKE_CONST_MOCK3(setOutputValue, bool(const Dsuid& dsuid, uint8_t value, ds::DateTime now), override);
  MAKE_CONST_MOCK4(setOutputValue, bool(const Dsuid& dsuid, ChannelType channel, double value, ds::DateTime now),
                   override);
  MAKE_CONST_MOCK3(callSceneMin, bool(const Dsuid& dsuid, int sceneId, ds::DateTime now), override);
  MAKE_CONST_MOCK2(setSceneLocalPrio, bool(const Dsuid& dsuid, int sceneId), override);
  MAKE_CONST_MOCK2(setOperationLock, void(const Dsuid& device, bool lock), override);
  MAKE_CONST_MOCK1(getState, DeviceState(const Dsuid& dsuid), override);
  MAKE_CONST_MOCK1(areaMember, std::vector<int>(const Dsuid& dsuid), override);

  MAKE_CONST_MOCK1(cacheStatus, DeviceCacheState(const Dsuid& dsuid), override);
};

} // namespace dss
