/*
    Copyright (c) 2009 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifdef HAVE_CONFIG_H
  #include "config.h"
#endif


#include "logger.h"
#include "base.h"
#include "datetools.h"

#include <iostream>
#include <boost/make_shared.hpp>
#include <boost/thread/mutex.hpp>

namespace dss {

aLogSeverity dsLogSeverityToLogSeverity(ds::LogSeverity x) {
  switch (x) {
    case ds::LogSeverity::PRINT:
      break;
    case ds::LogSeverity::FATAL:
      return lsFatal;
    case ds::LogSeverity::ERROR:
      return lsError;
    case ds::LogSeverity::WARNING:
      return lsWarning;
    case ds::LogSeverity::NOTICE:
      return lsNotice;
    case ds::LogSeverity::INFO:
      return lsInfo;
    case ds::LogSeverity::DEBUG:
      return lsDebug;
  }
  DS_FAIL_REQUIRE("Cannot convert ds log severity", x);
}

Logger::Logger() : m_defaultLogChannel("System") {}

Logger* Logger::getInstance() {
  // static pointer that leaks instance to avoid static
  // deinitialization order fiasco
  static Logger* s_instance = new Logger();
  return s_instance;
}

void Logger::log(std::string message, const aLogSeverity severity) {
  log(m_defaultLogChannel, std::move(message), severity);
}

void Logger::log(const LogChannel& channel, std::string message, const aLogSeverity severity) {
  auto dsSeverity = logSeverityToDsLogSeverity(severity);
  if (!channel.shouldLog(dsSeverity)) {
    return;
  }
  message.append(ds::str(" ", ds::logContext()));
  channel.log(dsSeverity, std::move(message));
}
}
