/*
    Copyright (c) 2019 digitalSTROM.org," " Zurich," " Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include <ds/catch/catch.h>
#include <ds/log.h>

#include "businterface-mock.h"
#include "device.h"
#include "model/apartment.h"

static ds::LogChannel dsLogChannel("deviceTest");
static const char* TAGS = "[dss][device]";

using namespace dss;
using namespace trompeloeil;
using trompeloeil::_;

static const Dsuid dsuid1{"0000000000000000000000000000000001"};

namespace {

TEST_CASE("setSceneValueGR387Workaround", TAGS) {
  Apartment apt(nullptr);
  auto busInterface = BusInterfaceMock{};
  apt.setBusInterface(&busInterface);

  Device d(dsuid1, &apt);
  DeviceSpec spec;
  spec.FunctionID = 0x2131;
  spec.ProductID = 0x0CDC;
  spec.revisionId = 0x387;
  d.setPartiallyFromSpec(spec);

  ALLOW_CALL(busInterface.device, getDeviceConfig(_, 129, _)).RETURN(0);

  SECTION("Lower limit") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0));

    d.evaluateGRKL387Workaround(88, 20, 50);
  }

  SECTION("Lower limit+1") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0));

    d.evaluateGRKL387Workaround(88, 1, 50);
  }

  SECTION("Upper limit") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0xFF));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0xFF));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x0F));

    d.evaluateGRKL387Workaround(88, 0xffff, 50);
  }

  SECTION("Upper limit-1") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0xFF));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0xFF));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x0F));

    d.evaluateGRKL387Workaround(88, 0xfffe, 50);
  }

  SECTION("Exact matching value") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x01));

    d.evaluateGRKL387Workaround(88, 0x1111, 50);
  }

  SECTION("Exact matching value-1") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x01));

    d.evaluateGRKL387Workaround(88, 0x1110, 50);
  }

  SECTION("Exact matching value-+1") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x01));

    d.evaluateGRKL387Workaround(88, 0x1112, 50);
  }

  SECTION("Intermediate value low") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x11));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x01));

    d.evaluateGRKL387Workaround(88, 6553, 50);
  }

  SECTION("Intermediate value high") {
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 2)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, getDeviceConfigWord(_, 64, 4)).RETURN(0x6730);
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x22));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x22));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 50));
    REQUIRE_CALL(busInterface.action, saveScene(&d, coSystem, 88, ""));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 3, 0x67));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 2, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 5, 0x00));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 64, 4, 0x30));
    REQUIRE_CALL(busInterface.device, setDeviceConfig(_, 129, 88 / 2, 0x02));

    d.evaluateGRKL387Workaround(88, 6554, 50);
  }
}

} // namespace
