/*
    Copyright (c) 2019 digitalSTROM.org," " Zurich," " Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include <ds/catch/catch.h>
#include <ds/clock-mock.h>
#include <ds/log.h>
#include <boost/range/algorithm/find.hpp>
#include <boost/range/algorithm/find_if.hpp>
#include <boost/range/algorithm/sort.hpp>

#include <trompeloeil.hpp>
#include "group.h"

using namespace std::literals;

static ds::LogChannel dsLogChannel("groupTest");
static const char* TAGS = "[dss][group]";

using namespace dss;
using namespace trompeloeil;
using trompeloeil::_;

static const Dsuid dsuid1{"0000000000000000000000000000000001"};
static const Dsuid dsuid2{"0000000000000000000000000000000002"};
static const Dsuid dsuid3{"0000000000000000000000000000000003"};

namespace {

TEST_CASE("setMaxSensorValue", TAGS) {
  AggregateSensorMax filter;
  auto clock = ds::ClockMocker{};

  SECTION("Calculate Max Value") {
    CHECK(filter.sensorPush(dsuid1, ds::Clock::now(), 3.0) == 3.0);
    CHECK(filter.sensorPush(dsuid2, ds::Clock::now(), 2.0) == 3.0);
    CHECK(filter.sensorPush(dsuid3, ds::Clock::now(), 3.2) == 3.2);
  }

  SECTION("Age Sensor") {
    CHECK(filter.sensorPush(dsuid1, ds::Clock::now(), 3.0) == 3.0);
    clock.advance(60s);
    CHECK(filter.sensorPush(dsuid2, ds::Clock::now(), 2.0) == 3.0);
    clock.advance(1h);
    CHECK(filter.sensorPush(dsuid3, ds::Clock::now(), 1.0) == 2.0);
    clock.advance(60s);
    CHECK(filter.sensorPush(dsuid1, ds::Clock::now(), 0.1) == 1.0);
  }
}

} // namespace
