/*
    Copyright (c) 2017 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/
#include "vdc-element-reader.h"
#include <ds/catch/catch.h>
#include <ds/log.h>
#include "messages/vdc-messages.pb.h"

static ds::LogChannel dsLogChannel("vdcElemenReaderTest");

static const char* TAGS = "[dss][dssVdcProtobuf]";

using namespace dss;
namespace {

// copy&paste from device-info-test.cpp
vdcapi::PropertyElement& addField(vdcapi::PropertyElement& e, const std::string& key) {
  auto&& x = *e.add_elements();
  x.set_name(key);
  return x;
}

void addField(vdcapi::PropertyElement& e, const std::string& key, int value) {
  addField(e, key).mutable_value()->set_v_int64(value);
};

void addField(vdcapi::PropertyElement& e, const std::string& key, const char* value) {
  addField(e, key).mutable_value()->set_v_string(value);
}

TEST_CASE("dssVdcEmptyElement", TAGS) {
  auto mockedVdcPropertyResponse = []() {
    vdcapi::Message message;
    message.set_type(vdcapi::VDC_RESPONSE_GET_PROPERTY);
    auto& vdcGetPropertyResponse = *message.mutable_vdc_response_get_property();
    auto& properties = *vdcGetPropertyResponse.add_properties();
    properties.set_name("properties");
    {
      auto& field0 = addField(properties, "field0");
      addField(field0, "intValue", 7);
      addField(field0, "strValue", "string");
      addField(field0, "nonInitialzedValue");
    }
    return message;
  }();

  SECTION("check field exists") {
    VdcElementReader reader(mockedVdcPropertyResponse.vdc_response_get_property().properties());
    CHECK(reader["properties"].isValid() == true);
    CHECK(reader["properties"]["field0"].isValid() == true);
    CHECK(reader["nonExistingField"].isValid() == false);
    CHECK(reader["properties"]["nonExistingField"].isValid() == false);

    CHECK(reader["properties"]["field0"]["strValue"].getValueAsString() == "string");
    CHECK(reader["properties"]["field0"]["intValue"].getValueAsInt() == 7);

    // generated code always returns true see PropertyValue::IsInitialized
    // CHECK(reader["properties"]["nonInitialzedValue"].getValue().IsInitialized() == false);
  }
}
} // namespace
