@0xf3010218842f7cfa; # unique file ID, generated by `capnp id`

using Cxx = import "/capnp/c++.capnp";
$Cxx.namespace("dss::view");

struct Option @0xec41d1eceb2c48a3 {
    title @0 :Text;
    showIf @1 :Text;
    enableIf @2 :Text;
    icon @3 :Text;
}
annotation option(enumerant) :Option;

struct Field @0xda915b5611333160 {
    # Field annotation providing validation and rendering intructions for UI editor.
    #
    # In addition to metadata provided at underlaying capnproto field, it provides:
    # * finer valid range: `min`, `max`, `step`, `optionsQuery`, ...
    # * showing / hiding the field: `showIf`
    # * enabling / disabling field: `enableIf`
    # * `unit`
    # * `watches`
    #
    # _Shown_ field enforces that its value is always valid.
    # Invalid shown value is automatically fixed (changed) into valid range.
    #
    # TODO(soon): add calculated values to eliminate copy&paste, provide read only , shown to UI

    title @0 :Text;
        # Translated field title.
        # If empty, underlaying capnproto field title is used.
        # TODO(someday): `titleExpr` for dynamic value

    min @1 :Text;
        # Minimal inclusive value for numeric types.
        # TODO(someday): `minExpr` for dynamic value

    max @2 :Text;
        # Maximal inclusive value for numeric types.
        # TODO(someday): `maxExpr` for dynamic value

    step @3 :Text;
        # Step for numeric types.
        # TODO(someday): `stepExpr` for dynamic value

    struct OptionsQuery {
        # Dynamic list of options generated from `forXIn` array.
        #
        # Pseudo code:
        # options = [];
        # for (x : eval(`forXIn`)) {
        #   options.push({id : eval(`id`), title : eval(`title`), icon : eval(`icon`));
        # }

        id @0 :Text; # Expression returning option id.
        title @1 :Text; # Expression returning translated option title.
        icon @2 :Text; # Expression returning option icon.
        forXIn @3 :Text; # Expression returning iterated array. Iterated item is in variable `x`.
    }
    optionsQuery @4 :OptionsQuery;
        # Independently of the underlaying field type,
        # restrict valid values to these options.

    # Shall icons be attached to options in options list?
    # Rendering hint if options list with icons needs special handling.
    iconInOptions @9 :Bool;

    enum Unit {
      none @0;
      second @1;
      percent @2;
    }
    unit @5 :Unit;
        # Field value unit.
        # Usually rendered at value right side.
        # Rendering hint about how to render the value.
        # E.g. (unit:seconds, min:0, max:600) can be rendered as minutes (0-10), seconds (0-60).

    showIf @6 :Text;
        # Expresion returning whether the fields shall be shown or not.
        # Evaluated after every model change.
        # Default "true".
        #
        # Shown field enforces that its value is valid.

    enableIf @7 :Text;
        # Expresion returning whether the fields shall be shown or not.
        # Evaluated after every model change.
        # Default "false".

    struct Watch {
        # `Do` callback executed whenever `watch` expresssion changes.

        watch @0 :Text;
            # The `watch` expresssion is evaluated on every model change
            # and returns the value that is watched.
            # The `watch` expression should not change its value
            # when executed multiple times with the same input.
            # That is, watchExpression should be idempotent.

        do @1 :Text;
            # Evaluated when the result from current `watch` evaluation
            # and previous `watch` evaluation are not equal and on init.
            # The `do` callback may change the model,
            # which may trigger other `watches` to fire.
            #
            # The `do` callback is evaluated with newWatchValue, oldWatchValue arguments:
            # function (newWatchValue, oldWatchValue) {
            #    `do`;
            # }

        doOnInit @2 :Bool = true;
            # At initialization, the `do` callback is evaluated to initialize the watcher.
            # In rare cases, this is undesirable.
            # `doOnInit == false` prevents the initialization.
    }
    watches @8 :List(Watch);
        # Watches relevant to the field.
}

annotation field(field) :Field;

struct View {
    # Rendering instructions to position fields into tabs, rows, etc
    #
    # TODO(WIP): this is WIP, needs polishing.

    struct Item {
      enum Type {
          field @0;
          # ?readOnlyField?
          # ?calculatedValue?
      }
      type @0 :Type;

      fieldId @1 :Text;

      # ?title? - overrides title in the field
    }

    struct Row {
      enum Type {

      }
      item @0 :Item;
      subItems @1: List(Item); # Items rendered horizontally
    }

    struct Tab {
        title @0 :Text;
        rows @1 :List(Row);
    }
    tabs @0 :List(Tab);
}
