/*
    Copyright (c) 2009, 2010 digitalSTROM.org, Zurich, Switzerland

    Author: Patrick Staehlin, futureLAB AG <pstaehlin@futurelab.ch>

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef WEBSERVER_H_
#define WEBSERVER_H_

#include <string>
#include <unordered_map>

#include <boost/shared_ptr.hpp>
#include <boost/thread/mutex.hpp>

#include <external/civetweb/civetweb.h>

#include "subsystem.h"

#include "websocket/simpleserver.h"

#define WEB_SESSION_TIMEOUT_MINUTES 3
#define WEB_SESSION_LIMIT 30
#define WEB_SOCKET_TIMEOUT_S 300 // 5min
namespace dss {

  class IDeviceInterface;
  class PropertyNode;
  class RestfulAPI;
  class RestfulRequest;
  class WebServerRequestHandlerJSON;
  class Session;
  class SessionManager;

  std::string extractToken(const char *_cookie);
  std::string extractAuthenticatedUser(const char *_header);
  std::string generateCookieString(const std::string &token);
  std::string generateRevokeCookieString();

  class WebServer : public Subsystem {
  public:
    typedef std::vector<std::unique_ptr<websocket::SimpleServer>> WebSocketServers;

  private:
    struct mg_context* m_mgContext;
    int m_TrustedPort;
    std::unordered_map<std::string, WebServerRequestHandlerJSON*> m_Handlers;
    boost::shared_ptr<RestfulAPI> m_pAPI;
    boost::shared_ptr<SessionManager> m_SessionManager;
    const WebSocketServers m_WebSocketServers;

  private:
    void setupAPI();
    void instantiateHandlers();
    void publishJSLogfiles();
    static RestfulRequest extractRequest (struct mg_connection* _connection,
                                          std::string _sublevel,
                                          const struct mg_request_info *_info);

  protected:
    int httpBrowseProperties(struct mg_connection* _connection,
                             RestfulRequest &request,
                             const std::string &trustedSetCookie);
    int jsonHandler(struct mg_connection *_connection, RestfulRequest &request, const std::string &trustedSetCookie,
        boost::shared_ptr<Session> _session, bool authenticated);
    int iconHandler(struct mg_connection* _connection,
                    RestfulRequest &request,
                    const std::string &trustedSetCookie);
    int logDownloadHandler(struct mg_connection *_connection, RestfulRequest &request,
        const std::string &trustedSetCookie, bool authenticated);

    int notifyHandler(struct mg_connection *connection, RestfulRequest &request);
    int apiHandler(struct mg_connection *connection, std::string &&path, const std::string &setCookie);

    static int httpRequestCallback(struct mg_connection* _connection,
                                   void *cbdata);
  protected:
    virtual void doStart();
  public:
    WebServer(DSS* _pDSS);
    ~WebServer();

    virtual void initialize();
    void setSessionManager(boost::shared_ptr<SessionManager> _pSessionManager);
    void sendToWebSockets(const Event &e);


  }; // WebServer

}

#endif /*WEBSERVER_H_*/
