#!/bin/bash

# Pulls translation files from database and transifex.
# Store them as
# * .po into /po folder for gettext in dss
# * .json into /locale-json folder for i18next translations
#
# The .json files could be created during build step,
# but i18next-conv is slow.
# It would slow down the build process considerably.

set -e

cd ${TOP_SRCDIR:-.}
. ./tx-env.sh


function convertJson {
    languageCode=`echo $language | tr '_' '-'`
    i18next-conv -l $1 -s po/${language}/$2.po -t locale-json/${languageCode}/$2.json
}

rm -rf po/*/*.po
rm -rf locale-json/*/*.json

# pull from transifex
if type "tx" > /dev/null; then
  tx pull -a
  tx pull --pseudo
fi

# pull vdc resources from database
for language in ${languages}; do
    echo "Update tx $language"
    dir=po/${language}
    mkdir -p ${dir}
    for gtin in ${gtins}; do
        echo -n ${gtin} ${language}
        domain=vdc-gtin-${gtin}
        file=${dir}/${domain}.po
        wget --quiet -O${file} "http://db.aizo.net/dss-tx-gtin.php?gtin=${gtin}&langcode=${language}" && echo ": OK" || echo ": FAIL"
        convertJson $language $domain
    done
    domain=vdc-common
    file=${dir}/${domain}.po
    echo -n ${file} ${language}
    wget --quiet -O${file} "http://db.aizo.net/dss-tx-common.php?langcode=${language}" && echo ": OK" || echo ": FAIL"
    convertJson $language $domain
    # transifex po files
    convertJson $language dss-mainline
done

git add --all 'po/*/*.po'
git add --all 'locale-json/*/*.json'
